<?php
/**
  * Required Configuration.
  * This file configures database access,
  * some required paths, and other config directives.
  *
  * @author Patricio Anguita
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @version $Revision: 1.29 $
  * @internal $Id: m2f_config.php.dist,v 1.29 2006/04/05 23:03:10 georgeocrawford Exp $
*/

// Choose to write the Mail2Forum log to a file (you can set the path to the file below), to a database entry, or to both
// Options are FILE, DATABASE or BOTH
define("_M2F_LOG_METHOD_", 'DATABASE');

// Path to error log file (Must be web-writable). Only necessary when _M2F_LOG_METHOD_ is FILE or BOTH (see above), and/or you are usig mbox as an input source
define("_M2F_LOG_FILE_", '/tmp/m2f.log');

// Debug level (PEAR_LOG_NOTICE for normal use ; PEAR_LOG_INFO for debugging ; PEAR_LOG_DEBUG for HEAVY debugging and variable dumps)
define("_M2F_LOG_LEVEL_", 'PEAR_LOG_NOTICE');

// Set forum & version (Only 'phpbb_204' as of now - should support any v2.0.x version)
define("_M2F_FORUM_VERSION_", 'phpbb_204');

// DB Config: Set this to 1 if you want to use phpbb's database login details
define("_M2F_USE_PHPBB_DB_", 1);

// If you don't want to use phpBB's database, fill these out.
// If your database is PostgreSQL, then change $type_db to 'postgres'
// Otherwise, leave it as is (the default for $type_db is 'mysql')
$type_db = 'mysql';
$user_db = 'db_user';
$pass_db = 'PASSWORD';
$name_db = 'db_name';
$host_db = 'your.mysql.host.com';

// Table prefix: Be careful not to put in here the same prefix as your phpBB installation
define("_M2F_TABLE_PREFIX_", 'm2f_');

// Use M2F Bundled Pear install (Recommended)
define("_M2F_USE_LOCAL_PEAR_", 1);

// If you want to use your global PEAR install, set up PEAR location:
$pear_root_path = '/usr/local/share/pear/';








/**********************************************************************************
*                                                                                 *
*                                                                                 *
*                          DO NOT EDIT BELOW THIS LINE!!!                         *
*                                                                                 *
*                                                                                 *
**********************************************************************************/


// Table Configuration
// -------------------

// Distribution list table
define("_M2F_DISTRIBUTIONLIST_TABLE_", _M2F_TABLE_PREFIX_ . 'distributionlist');

// Mailing list config table
define("_M2F_MAILINGLIST_TABLE_", _M2F_TABLE_PREFIX_ . 'mailinglist');

// Subscription info
define("_M2F_SUBSCRIPTION_TABLE_", _M2F_TABLE_PREFIX_ . 'subscription');

// Group Subscription info
define("_M2F_GROUP_SUBSCRIPTION_TABLE_", _M2F_TABLE_PREFIX_ . 'group_subscription');

// M2F Config table
define("_M2F_CONFIG_TABLE_", _M2F_TABLE_PREFIX_ . 'config');

// M2F Log table
define("_M2F_LOG_TABLE_", _M2F_TABLE_PREFIX_ . 'log');

// Distribution list config table 
define("_M2F_DISTRIBUTIONLIST_CONFIG_TABLE_", _M2F_TABLE_PREFIX_ . 'distributionlist_config');

// Cron table 
define("_M2F_CRON_TABLE_", _M2F_TABLE_PREFIX_ . 'cron');

// Sent emails Message IDs table
define("_M2F_MESSAGE_ID_TABLE_", _M2F_TABLE_PREFIX_ . 'message_id');

// Processed Message table - stores md5 hashes or message-ids
define("_M2F_MESSAGE_HASH_TABLE_", _M2F_TABLE_PREFIX_ . 'message_hash');

// Regexp Pattern Matching table
define("_M2F_PATTERN_MATCH_TABLE_", _M2F_TABLE_PREFIX_ . 'patterns');

// Link patterns to dist lists table
define("_M2F_DIST_LIST_PATTERNS_TABLE_", _M2F_TABLE_PREFIX_ . 'dist_list_patterns');

// Message store for moderation
define("_M2F_MODERATED_MSGS_TABLE_", _M2F_TABLE_PREFIX_ . 'moderated_msgs');


/***********************************************************************
	define all the bit values for user preferences.
	these should be Bitwise AND-ed with the user preference field. e.g. 
	e.g.   if ($user_pref & _M2F_USER_HTML_EMAIL_) { ....
***********************************************************************/

//bit 1 HTML or text emails / set = html, not set = text
define("_M2F_USER_HTML_EMAIL_", 1);

//bit 2 ignore attachments / set = send message only, not set = include them 
define("_M2F_USER_IGNORE_ATTACHMENTS_", 2);

//bit 3 attachments as links / set = urls, not set = attach the files 
define("_M2F_USER_ATTACHMENT_LINKS_", 4);

//bit 4 images in line / set = send images as disposition = inline, not set = attach them as files 
define("_M2F_USER_IMAGES_INLINE_", 8);

//bit 5 thumbnail images if available / set = send thumbnails, not set = full size 
define("_M2F_USER_THUMBNAIL_IMAGES_", 16);

// what is the maximum possible combination ?
define("_M2F_MAX_USER_PREF_", 31);


/***********************************************************************
	define all the bit values for pattern match/block target
	these should be Bitwise AND-ed with the target field. e.g. 
	e.g.   if ($target & _M2F_TARGET_BODY_) { ....
***********************************************************************/

// subject
define("_M2F_TARGET_SUBJECT_", 1);

// body
define("_M2F_TARGET_BODY_", 2);

// headers
define("_M2F_TARGET_HEADERS_", 4);

?>
