<?php
/**
  * Forum abstraction class.
  *
  * @author Patricio Anguita
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @version $Revision: 1.72 $
  * @internal $Id: m2f_forum.php,v 1.72 2007/02/12 17:03:06 georgeocrawford Exp $
*/
if (isset($_POST['m2f_root_path']) || isset($_GET['m2f_root_path']) || isset($_COOKIE['m2f_root_path'])) die ('Hacking attempt');

include_once (M2F_ROOT_PATH . 'm2f_common.php');
include_once (M2F_ROOT_PATH . 'includes/insert_post.php');

// Abstract forum class
class m2f_forum
{
	// Class factory function
	function factory($forum_version)
	{
		switch ($forum_version)
		{
			case 'phpbb_204':
				return new m2f_forum_phpbb_204();
			break;

			default:
		    $m2f_error_msg = "Cannot create m2f_forum for version: $forum_version";
		    m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__, __FILE__);
		    die;
		}
	}	
}

// Concrete forum classes
class m2f_forum_phpbb_204
{
	// See also forum_post declaration for phpbb204
		
	// Constructor
	function m2f_forum_phpbb_204()
	{
		if (!defined('FORUMS_TABLE')) define('FORUMS_TABLE', 'phpbb_forums');
		if (!defined('GROUPS_TABLE')) define('GROUPS_TABLE', 'phpbb_groups');
		if (!defined('USERS_TABLE')) define('USERS_TABLE', 'phpbb_users');
	}
	
	// Methods
	// Returns a detected user_id from a mail object
	function user_id_mail($email)
	{
		// We'll look at the senders email
		// and match it with a phpbb user.
		
		global $m2f_db;

		m2f_log(PEAR_LOG_DEBUG , "From: header: " . $email , __LINE__ , __FILE__ );
		
		// Finds all email addresses in a string
		$match_pattern = '/[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)*\@[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)+/i';

		if (preg_match($match_pattern, $email, $matches) )
		{
			$email = $matches[0];
			
			m2f_log(PEAR_LOG_INFO , "Detected user email: " . $email , __LINE__ , __FILE__ );
	
			// Select user_id where matching mail
			$sql = "SELECT user_id FROM " . USERS_TABLE . "
							WHERE user_email = '" . addcslashes($email,"'") . "'"; 
			
			$db_result =& $m2f_db->getCol($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			
			// Known email?
			if (count($db_result) == 1)
			{
				m2f_log(PEAR_LOG_INFO , "Detected user id: " . $db_result[0], __LINE__ , __FILE__ );
				return $db_result[0];
			}
		}
		
		// Anonymous user
		m2f_log(PEAR_LOG_INFO , "User id not detected, defaulting to Anonymous (user id " . _M2F_ANON_USERID_ . "): ", __LINE__ , __FILE__ );
		return _M2F_ANON_USERID_;
		
	}
	



	// Get all user info for use in outgoing emails 
	function get_user_details(&$forum_post) 
	{ 
		global $board_config, $m2f_db, $lang; 

		$sql = "SELECT username, user_sig, user_sig_bbcode_uid, user_allowsmile 
						FROM " . USERS_TABLE . " WHERE user_id = {$forum_post->user_data[user_id]}"; 
		$db_result = $m2f_db->getRow($sql); 
		check_db_result($db_result, $sql, __LINE__, __FILE__); 
		
		$username = ( $forum_post->user_id != ANONYMOUS ) ? $db_result['username'] : ( ( $forum_post->post_username != '' ) ? $forum_post->post_username : $lang['Guest'] );
		//$username = ( $forum_post->user_data[user_id] != ANONYMOUS ) ? $db_result['username'] : ( ( $forum_post->post_username != '' ) ? $forum_post->post_username : $lang['Guest'] );
		$user_sig = ( $forum_post->enable_sig && $db_result['user_sig'] != '' && $board_config['allow_sig'] ) ? $db_result['user_sig'] : ''; 
		
		return array($user_sig, $db_result['user_sig_bbcode_uid'], $db_result['user_allowsmile'], $username); 
	}
        
        
        
  function clean_text_part(&$string)
  {
		$string = preg_replace('~&#x([0-9a-f]+);~ei', 'chr(hexdec("\\1"))', $string);
		$string = preg_replace('~&#([0-9]+);~e', 'chr(\\1)', $string);
		// replace literal entities
		$trans_tbl = get_html_translation_table(HTML_ENTITIES);
		$trans_tbl = array_flip($trans_tbl);
		return strtr($string, $trans_tbl);
  }
        

	// Create text part of MIME email
	function set_text_body($message, $user_sig, $bbcode_uid, $user_sig_bbcode_uid, $user_allowsmile,
												 $enable_smilies, $enable_html, $username, $m2f_signature='', $msg_attachments) 
	{ 
		global $bbcode_tpl, $board_config;
		
	  $message = str_replace("''", "'", stripslashes(unprepare_message($message)));
	
		if ($enable_html)
		{
			// Convert line break and non-breaking space
			$message = str_replace("<br>","\n",$message); 
			$message = str_replace("&nbsp;"," ",$message); 
			
			// Remove the other HTML tags
			$message = strip_tags($message); 
		}
		
		$m2f_signature = strip_tags($m2f_signature); 

		$this->clean_text_part($user_sig);
		$this->clean_text_part($message);

   
   // Strip extended HTML entities
		$match = array('&#8216;', '&#8217;', '&#8220;', '&#8221;', '&#8364;');
		$repl = array("'", "'", '"', '"', '');
		$message = str_replace($match, $repl, $message);

		// Call the phpBB bbcode function and fool it into removing all of the bbcode
	  if ( $board_config['allow_bbcode'] ) 
	  {
	  	// The bbcode template isn't set at this point, so just pretend that it's set
			// so bbencode_second_pass() will replace the bbcode with null
			define("BBCODE_TPL_READY", true);
			$bbcode_tpl = array();

	  	if ($user_sig != '' && $user_sig_bbcode_uid != '' && strpos($user_sig, '[') !== false)
	  	{ 
		  	// First convert bbcode URLs, emails, and quotes to a text-friendly format
		  	$user_sig = preg_replace('#\[url=(.*?)\](.*?)\[/url\]#i', '$2 ($1)', $user_sig);
		  	$user_sig = preg_replace('#\[url\](.*?)\[/url\]#i', '$1', $user_sig);
		  	$user_sig = str_replace('[email]', '', $user_sig);
		  	$user_sig = str_replace('[/email]', '', $user_sig);
	    	$user_sig = bbencode_second_pass($user_sig, $user_sig_bbcode_uid); 
			} 
	   	if ($bbcode_uid != '' && strpos($message, '[') !== false) 
			{ 
		  	// First convert bbcode URLs, emails, and quotes to a text-friendly format
		  	$message = preg_replace('#\[url=(.*?)\](.*?)\[/url\]#i', '$2 ($1)', $message);
		  	$message = preg_replace('#\[url\](.*?)\[/url\]#i', '$1', $message);
		  	$message = preg_replace("#\[img:$bbcode_uid\](.*?)\[/img:$bbcode_uid\]#i", '[Image: $1 ]', $message);
		  	$message = str_replace('[email]', '', $message);
		  	$message = str_replace('[/email]', '', $message);
		  	$message = preg_replace("#\[quote:$bbcode_uid=\"(.*?)\"\]#", "\n$1 wrote:\n[quote]\n", $message);
		  	$message = str_replace("[quote:$bbcode_uid]", "\n[quote]\n", $message);
		  	$message = str_replace("[/quote:$bbcode_uid]", "\n[/quote]\n", $message);
		  	$message = preg_replace("#\[code.*?:$bbcode_uid\]#", "\nCode:\n", $message);
		  	$message = preg_replace("#\[/code.*?:$bbcode_uid\]#", "\n\n", $message);
		  	$message = preg_replace('#(\[\*:' . $bbcode_uid . '\][^[]*)#', "\t" . '$1' . "\n", $message); // make each list element appear on new line
 	   		$message = bbencode_second_pass($message, $bbcode_uid); 
 	   		$message = str_replace('&nbsp;', ' ', $message); // have to do this - the bbencode_second_pass_code function adds &nbsp; characters for formatting
 	  	}
 	  	
 	  	// Since we messed up the bbcode template, we need to restore it in case it's used later
 	  	global $bbcode_tpl;
			$bbcode_tpl = load_bbcode_template();
			$bbcode_tpl = prepare_bbcode_template($bbcode_tpl);
 	 	}
 	 	
 	 	// Remove the smilies
  	if ( $board_config['allow_smilies'] ) 
  	{ 
   		if ( $user_allowsmile && $user_sig != '' ) 
   		{ 
    		$user_sig = $this->remove_smilies($user_sig); 
   		} 
   		if ( $enable_smilies ) 
   		{ 
 	 			$message = $this->remove_smilies($message);
   		} 
  	} 



/**************************
    // Text link to attachments
    $attachment_links = (_M2F_SEND_ATTACHMENT_LINK_ == 1) ? $msg_attachments->text_links() : '';
    $attachment_links = empty($attachment_links) ? '' : $attachment_links . "\n\n";
**************************/



    
		$message = $this->quote($message);
		
		// This is some cool, but complicated code to allow a "text email" template. It would give
		// more flexibility for the outgoing email format, but should wait until after v1.0.
		/*		
		$script_name = preg_replace('/^\/?(.*?)\/?$/', '\1', trim($board_config['script_path']));
		$server_name = trim($board_config['server_name']);
		$server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
		$server_port = ($board_config['server_port'] <> 80) ? ':' . trim($board_config['server_port']) . '/' : '/';
		
		$site_link = $server_protocol . $server_name . $server_port . $script_name . '/index.php';

		$template->set_filenames(array(
			'text_body' => 'admin/m2f_text_email_body.tpl')
		);
		
		$template->assign_vars(array(
			'SITENAME' => $board_config['sitename'],
			'SITELINK' => $site_link,
			'MESSAGE' => $message,
			'USER_SIG' => $user_sig,
			'M2F_SIG' => $m2f_signature,
			'USERNAME' => $username
		));		

		$template->loadfile('text_body');
		$body = "";
		$uncompiled_body = $template->compile($template->uncompiled_code['text_body'], true, 'body');
		$uncompiled_body = str_replace('$this', '$template', $uncompiled_body);
		eval($uncompiled_body);
		*/
		
		$body = $message . ('' == $user_sig ? '' : "\n\n------------------------\n" . $user_sig) . "\n\n" . $m2f_signature . "\n\n" . $attachment_links;

 	return $body; 
  	
	} 

	// battarsa 16/04/2005
	// set quoting style like in email
	function quote($msg)
	{
		$lines = explode("\n", $msg);
		$level = 0;
		foreach ($lines as $line)
		{
			if ($line == '[quote]') $level++;
			elseif ($line == '[/quote]') $level--;
			else $new_lines[] = str_repeat("> ", $level) . $line;
		}
		$msg = implode("\n", $new_lines);
		return $msg;
	}
	
	// Similar to the phpBB smilies_pass function, but removes the smilies
	function remove_smilies($message)
	{
		static $orig, $repl;
		
		if (!isset($orig))
		{
			global $m2f_db;

			$orig = array();
			$repl = array();
	
			$sql = "SELECT code, emoticon FROM " . SMILIES_TABLE;
			$db_result = $m2f_db->getAll($sql);  // Could have used getCol, but smiley_sort() expects assoc array
	    check_db_result($db_result, $sql, __LINE__, __FILE__);
			$smilies = $db_result;
	
			if (count($smilies))
			{
				usort($smilies, 'smiley_sort');
				
				foreach ($smilies as $smile_row)
				{
					$smile = $smile_row['code'];
					// Only remove the smilies that begin and end with ':' for example ':grin:'
					if ($smile{0} == ':' && substr($smile, -1) == ':')
					{
						$orig[] = "/(?<=.\W|\W.|^\W)" . phpbb_preg_quote($smile, "/") . "(?=.\W|\W.|\W$)/";
						$repl[] = "[{$smile_row['emoticon']}]";
					}
				}
			}
		}
		
		$message = preg_replace($orig, $repl, ' ' . $message . ' ');
		$message = substr($message, 1, -1);
		
		return $message;
	}



	//Create html part of MIME email
	function set_html_body($message, $user_sig, $bbcode_uid, $user_sig_bbcode_uid, $user_allowsmile,
												 $enable_smilies, $username, $m2f_signature='', $msg_attachments, $template) 
	{ 
		global $board_config, $theme, $lang, $bbcode_tpl; 
		
	  $message = str_replace("''", "'", stripslashes(unprepare_message($message)));

	  // 
	  // Parse message and/or sig for BBCode if reqd 
	  // 

  	if ( $user_sig != '' && $user_sig_bbcode_uid != '' ) 
  	{ 
    	$user_sig = ( $board_config['allow_bbcode'] ) ? bbencode_second_pass($user_sig, $user_sig_bbcode_uid) : preg_replace('/\:[0-9a-z\:]+\]/si', ']', $user_sig); 
		} 
   	if ( $bbcode_uid != '' ) 
		{ 
  		$message = ( $board_config['allow_bbcode'] ) ? bbencode_second_pass($message, $bbcode_uid) : preg_replace('/\:[0-9a-z\:]+\]/si', ']', $message); 
  	} 

    m2f_log(PEAR_LOG_DEBUG , "Msg after bbencode pass: " , __LINE__ , __FILE__ , $message);
    
 	 	if ( $user_sig != '' ) 
  	{ 
   		$user_sig = make_clickable($user_sig); 
  	} 
 		if ($m2f_signature != '') 
 		{
 			$m2f_signature = make_clickable($m2f_signature);
			$m2f_signature = nl2br($m2f_signature);
  	}
  	$message = make_clickable($message); 
  	
  	// 
  	// Parse smilies 
  	// 
  	if ( $board_config['allow_smilies'] ) 
  	{ 
   		if ( $user_allowsmile && $user_sig != '' ) 
   		{ 
    		$user_sig = smilies_pass($user_sig); 
   		} 
   		if ( $enable_smilies ) 
   		{ 
    		$message = smilies_pass($message); 
   		} 
  	} 
  	
  	if ( $user_sig != '' ) 
  	{ 
   		$user_sig = '<br /><br /><hr /><br />' . str_replace("\r\n", "<br />\n", $user_sig); 
  	} 

  	$message = nl2br($message);

  	$message = str_replace('<img src="images', '<img src="http://'.$board_config['server_name'].$board_config['script_path'].'images', $message); 
  	$user_sig = str_replace('<img src="images', '<img src="http://'.$board_config['server_name'].$board_config['script_path'].'images', $user_sig); 
    



/**************************
    $attach_links = (_M2F_SEND_ATTACHMENT_LINK_ == 1) ? $msg_attachments->html_links() : '';
**************************/
		$template->set_filenames(array(
			'html_body' => 'admin/m2f_html_email_body.tpl')
		);

		$template->assign_vars(array(
			'T_HEAD_STYLESHEET' => $theme['head_stylesheet'],
			'T_BODY_BACKGROUND' => $theme['body_background'],
			'T_BODY_BGCOLOR' => '#'.$theme['body_bgcolor'],
			'T_BODY_TEXT' => '#'.$theme['body_text'],
			'T_BODY_LINK' => '#'.$theme['body_link'],
			'T_BODY_VLINK' => '#'.$theme['body_vlink'],
			'T_BODY_ALINK' => '#'.$theme['body_alink'],
			'T_BODY_HLINK' => '#'.$theme['body_hlink'],
			'T_TR_COLOR1' => '#'.$theme['tr_color1'],
			'T_TR_COLOR2' => '#'.$theme['tr_color2'],
			'T_TR_COLOR3' => '#'.$theme['tr_color3'],
			'T_TR_CLASS1' => $theme['tr_class1'],
			'T_TR_CLASS2' => $theme['tr_class2'],
			'T_TR_CLASS3' => $theme['tr_class3'],
			'T_TH_COLOR1' => '#'.$theme['th_color1'],
			'T_TH_COLOR2' => '#'.$theme['th_color2'],
			'T_TH_COLOR3' => '#'.$theme['th_color3'],
			'T_TH_CLASS1' => $theme['th_class1'],
			'T_TH_CLASS2' => $theme['th_class2'],
			'T_TH_CLASS3' => $theme['th_class3'],
			'T_TD_COLOR1' => '#'.$theme['td_color1'],
			'T_TD_COLOR2' => '#'.$theme['td_color2'],
			'T_TD_COLOR3' => '#'.$theme['td_color3'],
			'T_TD_CLASS1' => $theme['td_class1'],
			'T_TD_CLASS2' => $theme['td_class2'],
			'T_TD_CLASS3' => $theme['td_class3'],
			'T_FONTFACE1' => $theme['fontface1'],
			'T_FONTFACE2' => $theme['fontface2'],
			'T_FONTFACE3' => $theme['fontface3'],
			'T_FONTSIZE1' => $theme['fontsize1'] ? $theme['fontsize1'] : 10,
			'T_FONTSIZE2' => $theme['fontsize2'] ? $theme['fontsize2'] : 11,
			'T_FONTSIZE3' => $theme['fontsize3'] ? $theme['fontsize3'] : 12,
			'T_FONTCOLOR1' => '#'.$theme['fontcolor1'],
			'T_FONTCOLOR2' => '#'.$theme['fontcolor2'],
			'T_FONTCOLOR3' => '#'.$theme['fontcolor3'],
			'T_SPAN_CLASS1' => $theme['span_class1'],
			'T_SPAN_CLASS2' => $theme['span_class2'],
			'T_SPAN_CLASS3' => $theme['span_class3'],
			'MESSAGE' => $message,
		));		

		if (!empty($user_sig))
		{
			$template->assign_block_vars('switch_user_sig', array(
				'USER_SIG' => $user_sig,
				));
		}
		
		if (!empty($m2f_signature))
		{
			$template->assign_block_vars('switch_m2f_sig', array(
				'M2F_SIG' => $m2f_signature,
				));
		}
		
		$template->loadfile('html_body');
		$body = "";
		$uncompiled_body = $template->compile($template->uncompiled_code['html_body'], true, 'body');
		$uncompiled_body = str_replace('$this', '$template', $uncompiled_body);
		eval($uncompiled_body);

  	return $body; 
	} 

	// Posts a new msg on a given forum, on behalf of the poster, and returns the post_id
	function new_topic($forum_msg , $forum_id , $user_id, $post_id = NULL)
	{
		global $m2f_db;

		// Posts a new topic on the forum_id
		// $forum_msg is a forum_msg object
		
		// Clean up the msg
		$forum_msg->body = addslashes($forum_msg->body);
		$forum_msg->title = addslashes($forum_msg->title);
		$forum_msg->author = addslashes($forum_msg->author);

		// Get info for this user
		$sql = "SELECT * FROM " . USERS_TABLE . "
						WHERE user_id = '$user_id'";
		$user_details = $m2f_db->getRow($sql);
		check_db_result($user_details, $sql, __LINE__, __FILE__);

		// If user not found, it's a critical error
		if ($user_details == null) {
			$m2f_error_msg = 'Attempting to create new forum post, but user \'' . $user_id . '\' not found';
			m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
		}

		if($forum_msg->topic_id == 0) $forum_msg->topic_id = NULL;
		
		// If this is a new topic, look up the user's watch data from their profile.
		if ( is_null($forum_msg->topic_id) ) 
		{
  	  $user_notify = $user_details['user_notify'];
		}
		
		// Otherwise, grab the user's current watch status for this topic. This is convoluted
		// as the phpBB user_notification function will look this up again. However, it's better
		// to do this rather than try to alter phpBB.
		else 
		{
			$sql = "SELECT COUNT(*) FROM " . TOPICS_WATCH_TABLE . "
							WHERE user_id = '$user_id' AND
							      topic_id = '$forum_msg->topic_id'";
  		$db_result = $m2f_db->getOne($sql);
	    check_db_result($db_result, $sql, __LINE__, __FILE__);
  	  $user_notify = $db_result;
		}
		
		// Using netclectic's insert_post function
		m2f_log(PEAR_LOG_INFO , "Inserting new msg on forum id $forum_id , user id $user_id, author $forum_msg->author, user notify $user_notify", __LINE__ , __FILE__ );
    m2f_log(PEAR_LOG_DEBUG , "Forum msg" , __LINE__ , __FILE__ , $forum_msg);

    // See if the user wants to attach his/her sig:
  	$user_attachsig = $user_details['user_attachsig'];
    /* Insert post reference: 
     
    function m2f_insert_post (
    $message, 
    $subject, 
    $forum_id, 
    $user_id, 
    $user_name, 
    $user_attach_sig, 
    $topic_id = NULL, 
    $topic_type = POST_NORMAL, 
    $do_notification = false, 
    $notify_user = false, 
    $current_time = 0, 
    $error_die_function = '', 
    $html_on = 0, 
    $bbcode_on = 1, 
    $smilies_on = 1 )
    */


		if ($forum_msg->datetime != 0 && $forum_msg->datetime != '')
		{
		  $post_details = m2f_insert_post($forum_msg->body, $forum_msg->title, $forum_id, $user_id, $forum_msg->author, $user_attachsig, $forum_msg->topic_id, $post_id, POST_NORMAL, true, $user_notify, $forum_msg->datetime);
		}
		else
		{
		  $post_details = m2f_insert_post($forum_msg->body, $forum_msg->title, $forum_id, $user_id, $forum_msg->author, $user_attachsig, $forum_msg->topic_id, $post_id, POST_NORMAL, true, $user_notify);
		}
		
		// Return the post_id and topic id of this post
		return $post_details;
	}
	
	// Returns a mail to be sent to a mailing list, properly formatted
	function post_to_mail($forum_post)
	{
	  global $m2f_config, $m2f_db, $board_config, $template, $lang;

		$mail_msg = new mail_msg();
		$mail_msg->headers = array();

		// Determine email subject header	    
		// Check for blank post subject or "force subject match" setting
		if ($forum_post->mode == 'reply' && ($forum_post->title == '' || $m2f_config['_M2F_FORCE_SUBJECT_MATCH_'] == 1) )
		{
			$sql = "SELECT topic_title FROM " . TOPICS_TABLE . "
								WHERE topic_id = '$forum_post->topic_id'";
			$db_result = $m2f_db->getOne($sql); 
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			$mail_msg->headers['Subject'] = $m2f_config['_M2F_SUBJECT_REPLY_PREFIX_'] . ' ' . str_replace("''", "'", stripslashes(unprepare_message($db_result)));
		}
		else
		{
			$mail_msg->headers['Subject'] = str_replace("''", "'", stripslashes(unprepare_message($forum_post->title)));
		}
		
		// Get the M2F signature
		$m2f_signature = $m2f_config['_M2F_MAIL_MSG_SIGNATURE_'];
		if ($m2f_signature != '')
		{
			$post_url = $m2f_config['_M2F_INCLUDE_SIGNATURE_LINK_'] ? $this->create_post_url($forum_post->post_id) . "\n\n" : '';
			$m2f_signature = "\n\n\n" . $m2f_config['_M2F_SIGNATURE_SEPARATOR_'] . "\n\n" . $m2f_signature . "\n" . $post_url . $m2f_config['_M2F_SIGNATURE_SEPARATOR_'];
		}

		//get details of post - bbcode and user info, etc
		list($user_sig, $user_sig_bbcode_uid, $user_allowsmile, $username) = $this->get_user_details($forum_post); 

		// If the board has HTML off but the post has HTML 
		// on then we process it, else leave it alone 
		// 
		if ( !$board_config['allow_html'] ) 
		{ 
			if ( $user_sig != '') 
			{ 
				$user_sig = preg_replace('#(<)([\/]?.*?)(>)#is', "<\\2>", $user_sig); 
			} 
			if ( $forum_post->enable_html ) 
			{ 
				$forum_post->body = preg_replace('#(<)([\/]?.*?)(>)#is', "<\\2>", $forum_post->body); 
			} 
		} 
		
		if (!isset($template))
		{
			global $theme;
			$theme = setup_style($board_config['default_style']);
		}



/**************************
		if (_M2F_SEND_HTML_ == '1')
		{
/**************************/

			$mail_msg->html_body = $this->set_html_body(
																$forum_post->body, 
																$user_sig, 
																$forum_post->bbcode_uid,
																$user_sig_bbcode_uid, 
																$user_allowsmile, 
																$forum_post->enable_smilies, 
																$username, 
																$m2f_signature, 
																$forum_post->msg_attachments,
																$template // this must NOT be passed by reference or made global - love George!
															); 

/**************************
		}
/**************************/


		$mail_msg->body = $this->set_text_body(
												$forum_post->body, 
												$user_sig, 
												$forum_post->bbcode_uid, 
												$user_sig_bbcode_uid, 
												$user_allowsmile, 
												$forum_post->enable_smilies, 
												$forum_post->userdata['enable_html'], 
												$username, 
												$m2f_signature, 
												$forum_post->msg_attachments
											); 



		$mail_msg->attachments = $forum_post->msg_attachments;

																			 
		if ($forum_post->user_data['user_id'] != ANONYMOUS) 
		{ 
			// Find the user's email
			$sql = "SELECT user_email
				FROM  " . USERS_TABLE . " 
				WHERE user_id = " . $forum_post->user_data['user_id'];
  		$user_email = $m2f_db->getOne($sql);
	    check_db_result($user_email, $sql, __LINE__, __FILE__);
		  $mail_msg->headers['user_email'] = $user_email;
	    
//		  $mail_msg->headers['user_email'] = '"'.$forum_post->user_data['username'] . '"' . ' <'.$user_email.'>';
		  m2f_log(PEAR_LOG_INFO , "Setting user_email to " . $user_email, __LINE__ , __FILE__ );
		}

		$mail_msg->headers['username'] = isset($forum_post->user_data['username']) ? $forum_post->user_data['username'] : (isset($forum_post->post_username) ? $forum_post->post_username : '');
		m2f_log(PEAR_LOG_INFO , "Setting username to " . $mail_msg->headers['username'], __LINE__ , __FILE__ );

		// Return the mail object
		return $mail_msg;
	}
	


	// creates a link back to a topic for use in the <<forum_link>> tag
	function create_post_url($id)
	{
		global $board_config, $phpEx;

		$script_name = preg_replace('/^\/?(.*?)\/?$/', '\1', trim($board_config['script_path']));
		$script_name = ($script_name != '') ? $script_name . '/viewtopic.'.$phpEx : 'viewtopic.'.$phpEx;
		$server_name = trim($board_config['server_name']);
		$server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
		$server_port = ($board_config['server_port'] <> 80) ? ':' . trim($board_config['server_port']) . '/' : '/';
		
		return $server_protocol . $server_name . $server_port . $script_name . "?p=$id#$id";
	}

	// Returns the forum id of a forum post object
	function forum_id($forum_post)
	{
		return ($forum_post->forum_id);
	}


	// Gets information about forums which haven't been linked to mailing lists yet
	function get_forums()
  {
    global $m2f_db;
  
    $sql = "SELECT f.forum_id, f.forum_name
    				FROM " . FORUMS_TABLE . " f
    					LEFT JOIN " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d
    					USING (forum_id)
    				WHERE d.mailinglist_id is NULL";
    
    $db_result =& $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	  
    return $db_result;
  }
}

?>