<?php
/**
  * Handles inbound and outbound messages.
  *
  * @author Patricio Anguita
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @version $Revision: 1.153 $
  * @internal $Id: m2f_handler.php,v 1.153 2007/02/19 19:45:59 georgeocrawford Exp $
*/

if (isset($_POST['m2f_root_path']) || isset($_GET['m2f_root_path']) || isset($_COOKIE['m2f_root_path'])) die ('Hacking attempt');

include_once (M2F_ROOT_PATH . 'm2f_common.php');
include_once (M2F_ROOT_PATH . 'm2f_forum.php');
include_once (M2F_ROOT_PATH . 'm2f_mailinglist.php');
include_once (M2F_ROOT_PATH . 'm2f_attachments.php');
include_once (M2F_ROOT_PATH . 'm2f_cron.php');

// Forum message class
class forum_msg
{
	var $body, $title , $post_id , $attachments , $author , $topic_id , $user_id , $user_email , $user_data;

	// Returns raw text (Non bb'd)
	function to_text()
	{
		// For now it justs returns the body of the msg
		return $this->body;
	}

	// Returns bb2html text
	function to_html()
	{
		// For now it justs returns the body of the msg
		return $this->body;
	}
}

// Mail message class
class mail_msg
{
	// Body, headers (relational array), pear mail object
	var $body, $headers, $pear_mail, $param, $html_body, $attachments, $temp_body, $temp_headers;

  function mail_msg()
	{
		$this->attachments = new m2f_attachments;
	}

	// PEAR - compatible send
	function send($params, $recipients, $user_prefs = 21)
	{
		// Currently using PEAR mail functionality
		// http://pear.php.net/manual/en/package.mail.php

		$param = array();
		if ($params['_M2F_MAIL_BACKEND_'] == 'sendmail')
		{
			$param["sendmail_path"] = _M2F_SENDMAIL_PATH_;
			$param["sendmail_args"] = _M2F_SENDMAIL_ARGS_;
		}
		elseif ($params['_M2F_MAIL_BACKEND_'] == 'smtp')
		{
			$param["host"] = $params['_M2F_SMTP_HOST_'];
			$param["port"] = $params['_M2F_SMTP_PORT_'];
			$param["auth"] = ($params['_M2F_SMTP_AUTH_'] == 1 ? true : false );
			$param["username"] = $params['_M2F_SMTP_USERNAME_'];
			$param["password"] = $params['_M2F_SMTP_PASSWORD_'];
		}

		// If an HTML part was created, make a MIME message
		
		// new logic based on user prefs. Need multipart if
		// user has specifed html OR there are attachments and user has not requested links and has not said ignore attachments
		if ($this->attachments->requires_multipart($user_prefs))
		{
			// assumption at this point is there are attachments and the users does not want links
			$mime = new Mail_Mime("\n"); // create a new instance of the Mail_Mime class
			m2f_log(PEAR_LOG_INFO, "Creating MIME multipart for type : " . $user_prefs, __LINE__ , __FILE__ ); 

			// do we need to add text links?
			if ( !($user_prefs & _M2F_USER_IGNORE_ATTACHMENTS_) && 
						( 
							($user_prefs & _M2F_USER_ATTACHMENT_LINKS_) || 
							( 
								($user_prefs & _M2F_USER_HTML_EMAIL_) && 
								!($user_prefs & _M2F_USER_ATTACHMENT_LINKS_) && 
								($user_prefs & _M2F_USER_IMAGES_INLINE_)
							)
						)
					)
			{
				$this->temp_body = $this->body . $this->attachments->text_links();
			}
			else
			{
				$this->temp_body = $this->body;
			}

			// add the text			    
			$mime->setTxtBody($this->temp_body); // set our plaintext message

			// do we create the html BODY part?
			if ($user_prefs & _M2F_USER_HTML_EMAIL_) 
			{
				$attach_html = $this->attachments->html_end($user_prefs);
				
				// set our HTML message
				if ($attach_html != "")
				{
					 // removed the end of body and substitute attach_html (which contains a </body>
					 $attach_html = str_replace('</body>', $attach_html, $this->html_body); 
					 $mime->setHtmlBody($attach_html);
				}
				else
				{
					$mime->setHtmlBody($this->html_body);
				}	    	  
			}

			// add the attachments, specify whether to inline the images and if thumbnails are ok
			if ( !($user_prefs & _M2F_USER_ATTACHMENT_LINKS_) && !($user_prefs & _M2F_USER_IGNORE_ATTACHMENTS_) )
			{
				$this->attachments->add_outgoing(&$mime, 
							($user_prefs & _M2F_USER_IMAGES_INLINE_) AND ($user_prefs & _M2F_USER_HTML_EMAIL_), 
							($user_prefs & _M2F_USER_THUMBNAIL_IMAGES_));
			}

			// This builds the email message and returns it to $body. Call get() after all the text,
			// html and attachments are set. 
			$this->temp_body = $mime->get(); 

			if ($params['_M2F_MAIL_BACKEND_'] == 'mail')
			{
				$this->temp_body = str_replace("\r\n", "\n", $this->temp_body);
			}
				 
			$this->temp_headers = $mime->headers($this->headers); 
		
		} 
		else 
		{
			$this->temp_body = $this->body;
			$this->temp_headers = $this->headers;
			
			// do we need to add text links?
			if( !($user_prefs & _M2F_USER_IGNORE_ATTACHMENTS_) )
			{
				$this->temp_body .= $this->attachments->text_links();
			}
		}
			
		m2f_log(PEAR_LOG_INFO , "Instantiating PEAR mail object " , __LINE__ , __FILE__ );
		m2f_log(PEAR_LOG_DEBUG , "PEAR param" , __LINE__ , __FILE__ , $param);
		m2f_log(PEAR_LOG_DEBUG , "Config params" , __LINE__ , __FILE__ , $params);
		
		$this->pear_mail =& Mail::Factory($params['_M2F_MAIL_BACKEND_'], $param);
		if (PEAR::isError($this->pear_mail))
		{
		  $m2f_error_msg = 'Could not factory a PEAR mail object: ' . $this->pear_mail->GetMessage();
      m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
      return;
		}

		$list = isset($this->temp_headers['Bcc']) ? $this->temp_headers['Bcc'] : $recipients;
		m2f_log(PEAR_LOG_INFO, "Sending mail to recipients: $list" , __LINE__ , __FILE__);
		m2f_log(PEAR_LOG_DEBUG , "Mail object: " , __LINE__ , __FILE__ , $this);

		$send = $this->pear_mail->send($recipients , $this->temp_headers, $this->temp_body);

		if (PEAR::isError($send))
    {
      $m2f_error_msg = 'Could not send mail: '.$send->getMessage();
      m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
    }

		return ($send);
	}

	function send_by_pref($params, $recipients)
	{
		if ($params['_M2F_EXTERNAL_LIST_SERVER_'] == 1)
		{
			// sending to the list server only
			$recipient_list = $this->list_email;
			$this->headers['To'] = $this->list_email;
			
			// Send it using preference for the list server
			$this->send($params, $recipient_list, $recipients[$this->list_email]);			
		}
		else
		{
			// create the addresses for each combination of user prefs
			$addresslist = array();
			
			// invert the list .. can't use the array flip as we want duplicates'				
			foreach ($recipients as $address => $prefs)
			{
				$addresslist[$prefs][] = $address;
			}
			
			// for each combination that has a subscriber, post the mail
			for ($preference=0; $preference < _M2F_MAX_USER_PREF_; $preference++)
			{
				if (count($addresslist[$preference])>0) 
				{ 
					$batch_recipients = array_splice($addresslist[$preference], 0 , 500);
					$subscriber_list = implode(', ', $batch_recipients);
					
					// When using PHP's mail() function, put the subscribers on the Bcc line instead of passing
					// them in the recipients parameter. mail() will automatically create the TO headers from the
					// recipients
					if ($params['_M2F_MAIL_BACKEND_'] == 'mail')
					{
						$recipient_list = $this->list_email;
						$this->headers['Bcc'] = $subscriber_list;
					}
					else
					{
						// 	sending via smtp, mail are sent only to address in subscriber_list 
						//mailing-list address should be added to the list 
						$recipient_list = $subscriber_list . ", " . $this->list_email; 
						$this->headers['To'] = $this->list_email;
					}
					
					// Send it
					$this->send($params, $recipient_list, $preference);
				}
			}
		}
	}
}

// M2F Handler
class m2f_handler
{
	// We have a forum and mailinglist objects
	var $m2f_forum , $m2f_mailinglist , $m2f_cron;

	// Constructor
	function m2f_handler()
	{
		// Instatiate objects
		$this->m2f_forum = m2f_forum::factory(_M2F_FORUM_VERSION_);
		$this->m2f_mailinglist = m2f_mailinglist::factory('internal');
		$this->m2f_cron = new m2f_cron();
		$this->start_time = array_sum(explode(" ",microtime()));

		// Now just wait for an event..
	}


	// New forum msg event
	// The parameter is a forum_msg object
	function new_post_event($forum_post)
	{

		// Get a distribution list for this forum
		$distributionlist_id = $this->get_distributionlist_id(array('forum_id' => $forum_post->forum_id));
			
		// If the distribution list id is null, this forum isn't connected to a
		// mailing list, so just exit
		if ($distributionlist_id == null) {
  		m2f_log(PEAR_LOG_INFO , "No distribution list for this forum" , __LINE__ , __FILE__ );
			return;
		}

		// Get the distributionlist config
		m2f_log(PEAR_LOG_INFO , "Getting distributionlist config" , __LINE__ , __FILE__ );
		$lists = $this->get_distributionlists();
		$params = $this->get_distributionlist_configuration($distributionlist_id);

		if ($lists[$distributionlist_id]['status'] != 1)	// Distribution list is active
		{
  		m2f_log(PEAR_LOG_INFO , "Inactive distribution list $distributionlist_id" , __LINE__ , __FILE__ );
			return;
		}

		m2f_log(PEAR_LOG_NOTICE , "*********************************\n**       New forum post        **\n*********************************", __LINE__ , __FILE__ );

		$edited = FALSE;
		if ($forum_post->mode == 'editpost')
		{
			m2f_log(PEAR_LOG_INFO , "This is an edited post", __LINE__ , __FILE__ );
			if ($params['_M2F_RESEND_EDITED_POST_'] == 0) 
			{
				m2f_log(PEAR_LOG_INFO , "_M2F_RESEND_EDITED_POST_ is 0. Exiting.", __LINE__ , __FILE__ );
				return;
			}
			m2f_log(PEAR_LOG_INFO , "_M2F_RESEND_EDITED_POST_ is 1. Will send edited post to mailing list.", __LINE__ , __FILE__ );
			$edited = TRUE;
		}

		// Transform the post into a mail object..
		m2f_log(PEAR_LOG_INFO , "Transforming the post into a mail object" , __LINE__ , __FILE__ );
		$mail_msg = $this->m2f_forum->post_to_mail($forum_post);
		
		// Get the info of the distribution list
		$distributionlist_ids = $this->distributionlist_ids($distributionlist_id);

		// Get the mailing list id
		$mailinglist_id = $distributionlist_ids['mailinglist_id'];
		m2f_log(PEAR_LOG_INFO , "Got the mailing list id: $mailinglist_id" , __LINE__ , __FILE__ );

		if ($params['_M2F_MODERATED_LIST_OUT_'] == 1 && !($params['_M2F_MODERATOR_IGNORE_ADMIN_'] == 1 && $forum_post->user_data['user_level'] == 1) )
		{
			// Store it
			$result = $this->m2f_mailinglist->new_mail($params, $mail_msg, $mailinglist_id, $edited, $forum_post->post_id, $forum_post->topic_id, $forum_post->user_data['user_id'], 'out', TRUE);
			if (!empty($result))
			{
				list($mail_msg, $recipient_list) = $result;
				$this->store_message_out($mail_msg, $recipient_list, $params, $distributionlist_id, $mailinglist_id, $forum_post, $forum_post->forum_id, $forum_post->topic_id, $forum_post->post_id, $forum_post->user_data, 'mod_out');
			}
		}
		else
		{
			// Send it
			m2f_log(PEAR_LOG_INFO , "Sending mail to mailing list id " . $mailinglist_id, __LINE__ , __FILE__ );
			$this->m2f_mailinglist->new_mail($params, $mail_msg, $mailinglist_id, $edited, $forum_post->post_id, $forum_post->topic_id, $forum_post->user_data['user_id'], 'out');
		}
	}

	// New  mail event
	function new_mail_event($mail_msg, $distributionlist_id, $params, $force_moderate=FALSE)
	{
		global $m2f_db;

	  m2f_log(PEAR_LOG_INFO , "New incoming mail on distributionlist: $distributionlist_id " , __LINE__ , __FILE__ );

		// Get the info of the distribution list
    $distributionlist_ids = $this->distributionlist_ids($distributionlist_id);

		// Get the target forum_id from the distribution list table
		$forum_id = $distributionlist_ids['forum_id'];
	  $mailinglist_id = $distributionlist_ids['mailinglist_id'];
	  m2f_log(PEAR_LOG_INFO , "Using distributionlist_id $distributionlist_id, forum_id $forum_id, mailinglist_id $mailinglist_id" , __LINE__ , __FILE__ );

    // Transform the mail object into a post object
		$forum_msg = $this->m2f_mailinglist->mail_to_post($mail_msg, $params, $forum_id, $distributionlist_id);
			
		// Try to detect who sent this email
		if ($params['_M2F_ALLOW_GUEST_POSTING_'] != 0 && $params['_M2F_FORCE_GUEST_POSTING_'] == 1)
		{
			$user_id = _M2F_ANON_USERID_;
			m2f_log(PEAR_LOG_INFO , '_M2F_FORCE_GUEST_POSTING_ set, so force author to be anonymous ($user_id = ' . _M2F_ANON_USERID_ . ')', __LINE__ , __FILE__ );
		}
		else
		{
			$user_id = $this->m2f_forum->user_id_mail($mail_msg->headers['from']);
		}

		// User not detected, so set author field
		if ($user_id == _M2F_ANON_USERID_)
		{
			// Set author field
			if (trim($mail_msg->headers['from']) && $params['_M2F_SHOW_GUEST_EMAIL_'] > 0)
			{
				// This pattern searches for valid email addresses
				// Gives an array:
				// [0] = whole string
				// [1] = ' character if found at beginning
				// [2] = " character if found at beginning
				// [3] = Full Name (if ' was found at beginning)
				// [4] = Full Name (if " was found at beginning)
				// [5] = Full name (if neither was found at beginning)
				// [6] = email address (N.B. that email address in this form - "user@..." - is allowed (Yahoo Groups)
				$search_string = '/^(?:(?:\s*)(?:(\')|("))?(?:\s*)(?(1)([^\']+?)|(?(2)([^"]+?)|([^,]+?)))(?:\s*)(?(1)\'\s*<?|(?(2)"\s*<?|(?:\s+|\s*<))))?\s*<?([^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)*\@(?:(?:\.{3})|(?:[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)+)))>?/i';
 				if(preg_match($search_string, $mail_msg->headers['from'], $matches))
				{
					$email = trim($matches[6]);
					$full_name = trim($matches[3]) . trim($matches[4]) . trim($matches[5]);

					switch ($params['_M2F_SHOW_GUEST_EMAIL_'])
					{
						case 1: // username at example.com
							$forum_msg->author = str_replace('@', ' at ', $email);
							break;
						case 2: // Full Name
							if (empty($full_name))
							{
								// Find just the stuff before the @ sign
								$search_string = '/([^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)*)\@/';
								preg_match($search_string, $email, $more_matches);
								$full_name = $more_matches[1];
							}
							$forum_msg->author = $full_name;
							break;
						case 3: // Full Name (username @ example.com)
							$forum_msg->author = $full_name . ' (' . str_replace('@', ' at ', $email) .')';
							break;
					}
				}
				// if username is longer than database field, chop it
				$field_length = 25; // default for phpBB

				if ($m2f_db->dbsyntax == 'mysql')
				{
					$res  =& $m2f_db->query('SELECT post_username FROM ' . POSTS_TABLE);
					$info =  $m2f_db->tableInfo($res);
					$field_length = $info[0]['len'];
				}
				else if ($m2f_db->dbsyntax == 'pgsql')
				{
					$sql = "SELECT a.atttypmod AS len
									FROM pg_attribute a , pg_class c
									WHERE c.relname = '" . POSTS_TABLE . "'
									AND a.attrelid = c.oid 
									AND  a.attname = 'post_username'";
					$field_length = $m2f_db->getOne($sql) - 4; // take off 4 bytes added to varchar fields
					check_db_result($field_length, $sql, __LINE__, __FILE__);
				}

      	if (strlen($forum_msg->author) > $field_length) {
					$forum_msg->author = substr($forum_msg->author, 0, ($field_length - 3));
					$forum_msg->author .= "...";
				}
			}
			else
			{
				$forum_msg->author = 'Guest';
			}

			m2f_log(PEAR_LOG_INFO , "Post author set to: " . $forum_msg->author , __LINE__ , __FILE__ );
		}

		// If topic id is set, check for moved topic
		if ($forum_msg->topic_id)
		{
			$sql = "SELECT topic_status, topic_moved_id FROM " . TOPICS_TABLE . "
	      			WHERE topic_id = '{$forum_msg->topic_id}'";
			$db_result = $m2f_db->getRow($sql);
	    check_db_result($db_result, $sql, __LINE__, __FILE__);

			$topic_status = $db_result['topic_status'];
			if ($topic_status == TOPIC_MOVED)
			{
				// Topic was moved, so get new topic info
				$forum_msg->topic_id = $db_result['topic_moved_id'];

				$sql = "SELECT topic_status, forum_id FROM " . TOPICS_TABLE . "
		      			WHERE topic_id = '{$forum_msg->topic_id}'";
				$db_result = $m2f_db->getRow($sql);
		    check_db_result($db_result, $sql, __LINE__, __FILE__);

				$topic_status = $db_result['topic_status'];
				$forum_id = $db_result['forum_id'];
				m2f_log(PEAR_LOG_INFO , "Incoming email redirected to moved topic" , __LINE__ , __FILE__ );
			}
		}

		$reason = $this->check_post_permissions($forum_id, $topic_status, $user_id, $params, $forum_msg->topic_id);
		$mail_error = '';
		
		$userdata = get_userdata($user_id);

		if (_M2F_ENABLE_ATTACHMENTS_ == 1)
		{
			$mail_error = $mail_msg->attachments->validate_and_save_inbound_attachments($userdata, $forum_id);

			if ($mail_error != "")
			{
				$reason = 'attachments';
				m2f_log(PEAR_LOG_INFO , "Incoming email had attachment problems." , __LINE__ , __FILE__ );
			}
		}

		// Post is  not allowed. Take appropriate action
		if ($reason !== NULL && $params['_M2F_DISALLOW_GUEST_ACTION_'] != 'moderate')
		{
			$this->refuse_post($params, $mail_msg, $reason, $distributionlist_id, $mailinglist_id, $mail_error);
			return FALSE;
		}

		$mod_reason = NULL;
		if ($force_moderate === TRUE) $mod_reason = 'spam';
		if ($params['_M2F_MODERATED_LIST_IN_'] == 1 && !($params['_M2F_MODERATOR_IGNORE_ADMIN_'] == 1 && $userdata['user_level'] == 1)) $mod_reason = 'mod_in';
		if ($reason !== NULL && $params['_M2F_DISALLOW_GUEST_ACTION_'] == 'moderate') $mod_reason = $reason;
		
		if 	($mod_reason)
		{
			return $this->store_message_in($forum_msg, $forum_id, $user_id, $mail_msg, $userdata, $mailinglist_id, $distributionlist_id, $params, $mod_reason, $mail_error);
		}
		else
		{
			return $this->new_mail_event_proceed($forum_msg, $forum_id, $user_id, $mail_msg, $userdata, $mailinglist_id, $distributionlist_id, $params);
		}
	}

	function store_message_in(&$forum_msg, &$forum_id, &$user_id, &$mail_msg, &$userdata, &$mailinglist_id, &$distributionlist_id, $params, $mod_reason, $mail_error)
	{
		global $m2f_db, $board_config, $lang, $db;

		// remove attachment source from $mail_msg object - the attachments 
		// have already been upladed by now, so this will reduce database storage needed
		$num_attachments = count($mail_msg->attachments->attach_data);
		for ($i=0; $i<$num_attachments; $i++)
		{
			unset ($mail_msg->attachments->attach_data[$i]->mimepart);
		}
		
		$topic_id = empty($forum_msg->topic_id) ? 0 : $forum_msg->topic_id;
		unset($forum_msg->topic_id);
		
		$username = isset($userdata['username']) ? $lang['Guest'] : $userdata['username'];
		$subject = $forum_msg->title;
		$body = $forum_msg->body;
		
		$forum_msg = base64_encode(serialize($forum_msg));
		$mail_msg = base64_encode(serialize($mail_msg));
		$userdata = base64_encode(serialize($userdata));
		
		$id = $m2f_db->nextId(_M2F_MODERATED_MSGS_TABLE_, TRUE);
		$time = time();
		
		// enter a 'phantom' post to the posts table. This acts as a plcaeholder, reserving us
		// a post_id so that this topic will appear in the correct order in the forum. If this
		// message is authorised, we'll update the post. if it is rejected, we'll delete it.
		$sql = "INSERT INTO phpbb_posts (topic_id) VALUES (0)";
		$result = $db->sql_query($sql); // use phpBB's DB class for this - PEAR's insert_id scheme is too complicated
		$post_id = $db->sql_nextid();
		
		
		$sql = "INSERT INTO " . _M2F_MODERATED_MSGS_TABLE_ . "
							(id,
							distributionlist_id, 
							mailinglist_id, 
							forum_id, 
							topic_id, 
							post_id, 
							user_id, 
							forum_msg, 
							mail_msg, 
							userdata,
							direction,
							time,
							reason)
						VALUES
							('$id',
							'$distributionlist_id', 
							'$mailinglist_id', 
							'$forum_id', 
							'$topic_id', 
							'$post_id', 
							'$user_id', 
							'$forum_msg', 
							'$mail_msg', 
							'$userdata',
							'in',
							'$time',
							'$mod_reason')";
							
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
		m2f_log(PEAR_LOG_INFO , "Incoming email was stored for later moderation." , __LINE__ , __FILE__ );

		$this->notify_moderator($params, $board_config, $forum_id, $distributionlist_id, $username, $subject, $body, $mod_reason, $mail_error);

		return 'mod';
	}

	function store_message_out(&$mail_msg, &$recipient_list, &$params, &$distributionlist_id, &$mailinglist_id, &$forum_post, &$forum_id, &$topic_id, &$post_id, &$user_data, $mod_reason)
	{
		global $m2f_db, $board_config, $lang;
		
		$username = isset($user_data['username']) ? $user_data['username'] : $lang['Guest'];
		$subject = $forum_post->title;
		$body = $forum_post->body;

		$forum_post = base64_encode(serialize($forum_post));
		$mail_msg = base64_encode(serialize($mail_msg));
		$user_data = base64_encode(serialize($user_data));
		$recipient_list = base64_encode(serialize($recipient_list));

		$id = $m2f_db->nextId(_M2F_MODERATED_MSGS_TABLE_, TRUE);
		$time = time();
		
		$sql = "INSERT INTO " . _M2F_MODERATED_MSGS_TABLE_ . "
							(id,
							distributionlist_id, 
							mailinglist_id, 
							forum_id, 
							topic_id, 
							user_id, 
							forum_msg, 
							mail_msg, 
							userdata,
							recipient_list,
							direction,
							time,
							reason)
						VALUES
							('$id',
							'$distributionlist_id', 
							'$mailinglist_id', 
							'$forum_id', 
							'$topic_id', 
							'$user_id', 
							'$forum_post', 
							'$mail_msg', 
							'$user_data',
							'$recipient_list',
							'out',
							'$time',
							'$mod_reason')";
							
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
		m2f_log(PEAR_LOG_INFO , "Outgoing email was stored for later moderation." , __LINE__ , __FILE__ );

		$this->notify_moderator($params, $board_config, $forum_id, $distributionlist_id, $username, $subject, $body, $mod_reason);
	}
	
	function notify_moderator($params, $board_config, $forum_id, $distributionlist_id, $username, $subject, $body, $mod_reason, $mail_error = '')
	{
		m2f_log(PEAR_LOG_INFO , "Notifying moderator" , __LINE__ , __FILE__ );
		
		switch ($params['_M2F_MODERATOR_NOTIFY_'])
		{
			case 'none':
				$recipients = array();
				break;
			case 'board_admin':
				$recipients = array($board_config['board_email']);
				break;
			case 'moderators':
				$recipients = $this->get_moderator_list($forum_id);
				break;
			case 'other_address':
				$recipients = array($params['_M2F_MODERATOR_NOTIFY_EMAIL_']);
				break;
		}
		
		$recipients = implode(', ', array_unique($recipients));
		
		if ($recipients)
		{
			$mod_reason_desc = array(
				'mod_in' => 'Inbound moderation is on for all messages',
				'mod_out' => 'Outbound moderation is on for all messages',
				'spam' => 'Message matched a spam block pattern',
				'attachments' => 'Message contained one or more illegal attachments',
				'forum_locked' => 'Forum was locked',
				'topic_locked' => 'Topic was locked',
				'guest' => 'Guest posts not allowed',
				'banned' => 'Message from banned user',
				'permission' => 'User didn\'t have permission to post'
			);

			$admin_mail = new mail_msg;
			
			$admin_mail->headers['From'] = '"' . $this->get_mailinglist_name($distributionlist_id) . '" <' . $this->get_mailinglist_address($distributionlist_id) . '>';		
			$admin_mail->headers['Subject'] = 'New moderated email';
			$admin_mail->headers['To'] = $recipients;
			
			$script_name = preg_replace('/^\/?(.*?)\/?$/', '\1', trim($board_config['script_path']));
			$server_name = trim($board_config['server_name']);
			$server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
			$server_port = ($board_config['server_port'] != 80) ? ':' . trim($board_config['server_port']) . '/' : '/';

			$admin_mail->body = "There is a new message waiting in the moderation control panel for the " . $board_config['sitename'] . " forum. Click the link below to visit the forum.\n";
			$admin_mail->body .= $server_protocol . $server_name . $server_port . $script_name . "/index.php\n\n\n";
			$admin_mail->body .= "Message Details:\n******************************\nAuthor: $username\nSubject: $subject\nReason for moderation: {$mod_reason_desc[$mod_reason]}. $mail_error\n\n\nBody:\n******************************\n\n$body";
		
			$send = $admin_mail->send($params, $recipients);
			m2f_log(PEAR_LOG_INFO , "Email alert was sent to designated moderators: " . $recipients , __LINE__ , __FILE__ );
		}
		else
		{
			m2f_log(PEAR_LOG_INFO , "Email alert not sent to designated moderators, recipient list was empty." , __LINE__ , __FILE__ );
		}
	}
		
	function get_moderator_list($forum_id)
	{
		global $m2f_db;

		$sql = "SELECT u.user_email FROM " . USER_GROUP_TABLE . " ug, " . USERS_TABLE . " u, " . AUTH_ACCESS_TABLE . " aa
						WHERE aa.forum_id = '$forum_id'
						AND aa.auth_mod = '1'
						AND aa.group_id = ug.group_id
						AND ug.user_id = u.user_id
						AND ug.user_pending = '0'";
		$db_result = $m2f_db->getCol($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
    return $db_result;
	}
	
	function new_post_event_proceed(&$mail_msg, &$recipient_list)
	{
		$params = empty($params) ? $this->get_distributionlist_configuration($distributionlist_id) : $params;
		$mail_msg->send_by_pref($params, $recipient_list);
	}
	
	function new_mail_event_proceed(&$forum_msg, &$forum_id, &$user_id, &$mail_msg, &$userdata, &$mailinglist_id, &$distributionlist_id, $params = array(), $is_moderated = FALSE, $post_id = FALSE)
	{
		$params = empty($params) ? $this->get_distributionlist_configuration($distributionlist_id) : $params;

		// Post the object in the forum
		$post_details = $this->m2f_forum->new_topic($forum_msg, $forum_id, $user_id, $post_id);

		$topic_id = $post_details['topic_id'];
		$post_id = $post_details['post_id'];
		$bbcode_uid = $post_details['bbcode_uid'];
		$message = str_replace("''", "'", $post_details['message']);

		// update db with any attachments
		if (_M2F_ENABLE_ATTACHMENTS_ == 1)
		{
		  $mail_msg->attachments->update_post($post_id, $topic_id, $user_id);
		}

		if ($params['_M2F_EXTERNAL_LIST_SERVER_'] == 0)
		{
			m2f_log(PEAR_LOG_INFO , "Resending incoming mail to the mailing list" , __LINE__ , __FILE__ );

			global $board_config;
			
			$m2f_forum_post 								= new m2f_forum_post();
			$m2f_forum_post->title 					= $mail_msg->headers['subject'];
			$m2f_forum_post->topic_id 			= $topic_id;
			$m2f_forum_post->post_id 				= $post_id;
			$m2f_forum_post->forum_id 			= $forum_id;
			$m2f_forum_post->body 					= $message;
			$m2f_forum_post->bbcode_uid 		= $bbcode_uid; 
			$m2f_forum_post->user_data 			= ($userdata === FALSE ? 
																					array('user_id' => _M2F_ANON_USERID_,
																								'user_attachsig' => 0,
																								'user_allowhtml' => $board_config['allow_html']
																								) : 
																					$userdata);
			$m2f_forum_post->mode 					= empty($forum_msg->topic_id) ? 'newtopic' : 'reply';
			$m2f_forum_post->post_username 	= $forum_msg->author;
			$m2f_forum_post->enable_sig			= $userdata['user_attachsig'];
			$m2f_forum_post->user_id				= $userdata['user_id'];


			if (_M2F_ENABLE_ATTACHMENTS_ == 1)
			{
				$m2f_forum_post->msg_attachments = $mail_msg->attachments;
			}

			$new_mail_msg = $this->m2f_forum->post_to_mail($m2f_forum_post);

			if (_M2F_FORUM_POST_SIGNATURE_ != '')
			{
				// remove the post signature
				$sig = preg_quote(_M2F_FORUM_POST_SIGNATURE_, '/');
				$search = '/^([\n\r\s>]|\<br \/\>|\<br\>)*^.*' . $sig . '.*$([\n\r\s>]|\<br \/\>|\<br\>)/m';
				$new_mail_msg->html_body = preg_replace($search , '', $new_mail_msg->html_body);
				$new_mail_msg->body = preg_replace($search , '', $new_mail_msg->body);
			}

			// Preserve the From header from the original mail
			$new_mail_msg->headers['from'] = $mail_msg->headers['from'];

			// Preserve reply-to header to weed out Yahoo Groups resends
			if (!empty($mail_msg->headers['reply-to'])) $new_mail_msg->headers['reply-to'] = $mail_msg->headers['reply-to'];

			m2f_log(PEAR_LOG_DEBUG, "Reconstituted HTML mail message", __LINE__, __FILE__, $new_mail_msg);

			// Send mails
			$this->m2f_mailinglist->new_mail($params, $new_mail_msg, $mailinglist_id, FALSE, $post_id, $topic_id, $user_id, 'in');
		}

		return $is_moderated ? 'mod' : true;
	}

	function check_post_permissions ($forum_id, $topic_status, $user_id, $params, $topic_id=NULL) // returns NULL if no error, $reason if error
	{
		global $m2f_db;

		// Check for locked forum and topic
		$sql = "SELECT forum_status FROM " . FORUMS_TABLE . "
      			WHERE forum_id = '$forum_id'";
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    if ($db_result == FORUM_LOCKED)
    {
    	$reason = 'forum_locked';
			m2f_log(PEAR_LOG_INFO , "Incoming email not allowed to locked forum" , __LINE__ , __FILE__ );
			return $reason;
    }
    else if ($topic_status == TOPIC_LOCKED)
    {
			$reason = 'topic_locked';
			m2f_log(PEAR_LOG_INFO , "Incoming email not allowed to locked topic" , __LINE__ , __FILE__ );
			return $reason;
    }

		// If guest posting is not allowed, refuse an anonymous email
		if ($user_id == _M2F_ANON_USERID_ && ($params['_M2F_ALLOW_GUEST_POSTING_'] == '0'))
		{
			$reason = 'guest';
			m2f_log(PEAR_LOG_INFO , "Incoming email from guest not allowed" , __LINE__ , __FILE__ );
			return $reason;
		}

		// Check for the sender in the ban table
		$sql = "SELECT ban_userid FROM " . BANLIST_TABLE . "
						WHERE ban_userid = '$user_id'";
		$db_result = $m2f_db->getOne($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);

		if ($db_result == $user_id)
		{
			$reason = 'banned';
			m2f_log(PEAR_LOG_INFO , "Incoming email from banned user" , __LINE__ , __FILE__ );
			return $reason;
		}

		if ($params['_M2F_IGNORE_SECURITY_SETTINGS_'] != 1) // only do a phpBB security check if we're supposed to
		{
			// extra check right here to see if they're disabled... because setting session_logged_in to true ASSUMES that they're not
			if ($user_id != _M2F_ANON_USERID_)
			{
				$sql = "SELECT user_active FROM " . USERS_TABLE . "
									 WHERE user_id = '$user_id'";
				$db_result = $m2f_db->getOne($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);

				if($db_result == 0)
				{
					// user is disabled - get outta here!!
					$reason = 'permission';
					m2f_log(PEAR_LOG_INFO , "Incoming email from inactive (disabled) user" , __LINE__ , __FILE__ );
					return $reason;
				}
			}

			// See if the user has post permission for the forum
			// In order to call the phpBB permission function, we have to fake some data
			$userdata = array();
			$userdata['session_logged_in'] = true;
			$userdata['user_id'] = $user_id;
			$sql = "SELECT user_level FROM " . USERS_TABLE . "
							WHERE user_id = '$user_id'";
			$db_result = $m2f_db->getOne($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);

			$userdata['user_level'] = $db_result;

			$is_auth = auth(AUTH_ALL, $forum_id, $userdata);

			if (!($is_auth['auth_post'] || ($is_auth['auth_reply'] && !empty($topic_id))))
			{
				$reason = 'permission';
				m2f_log(PEAR_LOG_INFO , "Incoming email from user without post permission" , __LINE__ , __FILE__ );
				return $reason;
			}
		}

		return NULL;
	}

	function refuse_post($params, $mail_msg, $reason, $distributionlist_id, $mailinglist_id, $mail_error = '')
	{
		global $board_config;

		// dirty hack to temporarily change setting for emails which are too large - never inform admin
		if ($reason == 'size' && $params['_M2F_DISALLOW_GUEST_ACTION_'] == 'bounce_and_admin') $params['_M2F_DISALLOW_GUEST_ACTION_'] = 'bounce';
		if ($reason == 'size' && $params['_M2F_DISALLOW_GUEST_ACTION_'] == 'admin') $params['_M2F_DISALLOW_GUEST_ACTION_'] = 'ignore';

		// dirty hack to temporarily change setting for emails which fail the spam check - never bounce mail
		if ($reason == 'spam' && $params['_M2F_DISALLOW_GUEST_ACTION_'] == 'bounce_and_admin') $params['_M2F_DISALLOW_GUEST_ACTION_'] = 'admin';
		if ($reason == 'spam' && $params['_M2F_DISALLOW_GUEST_ACTION_'] == 'bounce') $params['_M2F_DISALLOW_GUEST_ACTION_'] = 'ignore';

		if ( ($params['_M2F_DISALLOW_GUEST_ACTION_'] == 'admin') || ($params['_M2F_DISALLOW_GUEST_ACTION_'] == 'bounce_and_admin') )
		{
			$admin_address = $params['_M2F_ADMIN_EMAIL_ADDRESS_'];
			if ($admin_address == '')
			{
				$admin_address = $board_config['board_email'];
			}

			$admin_body = "The following message from " . $mail_msg->headers['from'] . " was not authorized for entry in the $board_config[sitename] forum.\n\n";
			switch ($reason)
			{
				case 'guest':
				$admin_body .= "Reason: The email was not from a registered user, and guest posts are not allowed in this forum.\n\n";
				break;

				case 'banned':
				$admin_body .= "Reason: The email was from a banned user.\n\n";
				break;

				case 'permission':
				$admin_body .= "Reason: The email was from a user that did not have sufficient permissions to post in this forum.\n\n";
				break;

				case 'forum_locked':
				$admin_body .= "Reason: The forum is locked, forum_id = $forum_id.\n\n";
				break;

				case 'attachments':
				$admin_body .= "Reason: There were problems with the attachments: " . $mail_error . " \n\n";
				break;

				case 'topic_locked':
				$admin_body .= "Reason: The topic is locked, topic_id = {$forum_msg->topic_id}.\n\n";
				break;

				case 'spam':
				$admin_body .= "Reason: The incoming mail matched a spam-block pattern:\n    [" . $mail_error . "]\n\n";
				break;
			}
			$admin_body .= "Distribution List ID:\n$distributionlist_id\n\n";
			$admin_body .= "Mailing List ID:\n$mailinglist_id\n\n";
			$admin_body .= "From:\n" . $mail_msg->headers['from'] . "\n\n";
			$admin_body .= "To:\n" . $mail_msg->headers['to'] . "\n\n";
			$admin_body .= "Subject:\n" . $mail_msg->headers['subject'] . "\n\n\n";
			$admin_body .= "****************************************\n\n";
			$admin_body .= $mail_msg->body;
			$admin_body .= "\n\n****************************************\n\nOriginal Mail Headers:\n\n";
			ob_start();
			print_r($mail_msg->headers);
			$admin_body .= ob_get_contents();
			ob_end_clean();
		}

		if ( ($params['_M2F_DISALLOW_GUEST_ACTION_'] == 'bounce') || ($params['_M2F_DISALLOW_GUEST_ACTION_'] == 'bounce_and_admin') )
		{
			if ($params['_M2F_BOUNCE_INCLUDE_LINK_'] == '1')
			{
				$script_name = preg_replace('/^\/?(.*?)\/?$/', '\1', trim($board_config['script_path']));
				$server_name = trim($board_config['server_name']);
				$server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
				$server_port = ($board_config['server_port'] != 80) ? ':' . trim($board_config['server_port']) . '/' : '/';

				$link = "Visit this URL to register: ";
				$link .= $server_protocol . $server_name . $server_port . $script_name . "/index.php\n\n\n";
			}

			$bounce_body = "The following message from " . $mail_msg->headers['from'] . " was not authorized for entry in the $board_config[sitename] forum. \n\n";
			switch ($reason)
			{
				case 'guest':
				$bounce_body .= "Posts can only be made by registered users of the forum.\n\n" . $link;
				break;

				case 'banned':
				$bounce_body .= "Posts are not allowed from banned users of the forum.";
				break;

				case 'permission':
				$bounce_body .= "Posts are only allowed from users with permission to post in the forum.";
				break;

				case 'forum_locked':
				$bounce_body .= "Posts are not allowed to locked forums.";
				break;

				case 'attachments':
				$bounce_body .= "Reason: There were problems with the attachments [" . $mail_error . "] ";
				break;

				case 'topic_locked':
				$bounce_body .= "Posts are not allowed to locked topics.";
				break;

				case 'size':
				$bounce_body .= "The email was larger than the maximum specified import size for this forum.";
				break;

			}

			$bounce_body .= "\n\nFrom:\n" . $mail_msg->headers['from'] . "\n\n";
			$bounce_body .= "To:\n" . $mail_msg->headers['to'] . "\n\n";
			$bounce_body .= "Subject:\n" . $mail_msg->headers['subject'] . "\n\n";
			$bounce_body .= "****************************************\n\n";
			$bounce_body .= $mail_msg->body;
		}

		switch($params['_M2F_DISALLOW_GUEST_ACTION_'])
		{
			case 'bounce':
				$bounce_mail = new mail_msg;
				$bounce_mail->headers['From'] = $mail_msg->headers['to'];
				$bounce_mail->headers['Subject'] = "Returned Mail: " . $mail_msg->headers['subject'];
				$bounce_mail->headers['To'] = $mail_msg->headers['from'];
				$recipients = $mail_msg->headers['from'];
				$bounce_mail->body = $bounce_body;
				$send = $bounce_mail->send($params, $recipients);
				m2f_log(PEAR_LOG_INFO , "Incoming email was bounced back to sender" , __LINE__ , __FILE__ );
				break;

			case 'admin':
				$admin_mail = new mail_msg;
				$admin_mail->headers['From'] = $mail_msg->headers['to'];
				$admin_mail->headers['Subject'] = "Unauthorized Mail: " . $mail_msg->headers['subject'];
				$admin_mail->headers['To'] = $admin_address;
				$recipients = $admin_address;
				$admin_mail->body = $admin_body;
				$send = $admin_mail->send($params, $recipients);
				m2f_log(PEAR_LOG_INFO , "Incoming email was re-routed to admin" , __LINE__ , __FILE__ );
				break;

			case 'bounce_and_admin':
				$admin_mail = new mail_msg;
				$admin_mail->headers['From'] = $mail_msg->headers['to'];
				$admin_mail->headers['Subject'] = "Unauthorized Mail: " . $mail_msg->headers['subject'];
				$admin_mail->headers['To'] = $admin_address;
				$recipients = $admin_address;
				$admin_mail->body = $admin_body;
				$send = $admin_mail->send($params, $recipients);

				$bounce_mail = new mail_msg;
				$bounce_mail->headers['From'] = $mail_msg->headers['to'];
				$bounce_mail->headers['Subject'] = "Returned Mail: " . $mail_msg->headers['subject'];
				$bounce_mail->headers['To'] = $mail_msg->headers['from'];
				$recipients = $mail_msg->headers['from'];
				$bounce_mail->body = $bounce_body;
				$send = $bounce_mail->send($params, $recipients);
				m2f_log(PEAR_LOG_INFO , "Incoming email was bounced back to sender and routed to admin" , __LINE__ , __FILE__ );
				break;

			case 'delete':
				m2f_log(PEAR_LOG_INFO , "Incoming email ignored" , __LINE__ , __FILE__ );
				break;
		}
	}

	// Guesses mailinglist_id from the TO header of a mail msg
	function get_mailinglist_id($mail_msg)
	{
		global $m2f_db;

		m2f_log(PEAR_LOG_INFO , "Trying to detect mailinglist id" , __LINE__ , __FILE__ );

		// Finds all email addresses in a string
		$match_pattern = '/[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)*\@[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)+/i';

		if ( preg_match($match_pattern, $mail_msg->headers['to'], $matches) )
		{
			$to_header = $matches[0];
		}
		else
		{
			$to_header = $mail_msg->headers['to'];
		}

		// Get the header, and clean it up
		$to_header = addslashes($to_header);

		// Look it up on the DB
		$sql = "SELECT mailinglist_id FROM " . _M2F_MAILINGLIST_TABLE_ . "
						WHERE list_email = '$to_header'";
		$db_result = $m2f_db->getOne($sql); // We return the first match only!
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		if ($db_result)
		{
			m2f_log(PEAR_LOG_INFO , "Detected mailinglist id:" . $db_result, __LINE__ , __FILE__ );
		}
		else
		{
			m2f_log(PEAR_LOG_INFO , 'No mailinglist ID was detected', __LINE__ , __FILE__ );
		}

		return $db_result;
	}

	// Returns the ids of a distribution list id
	function distributionlist_ids($distributionlist_id)
	{
		global $m2f_db;

		// Return a relational array $distribution_ids['forum_id'] ,
		// $distribution_ids['mailinglist_id'] , etc
		$sql = "SELECT * FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
						WHERE distributionlist_id = '$distributionlist_id'";
		$db_result =& $m2f_db->getRow($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		return $db_result;
	}

	// Returns the distribution list id, given other id (forum, mail etc)
	// Parameter array should be $id_array['forum_id'] or $id_array['mailinglist_id'], etc
	function get_distributionlist_id($field_array)
	{
		global $m2f_db;
		// Find the distribution list ID that matches the fields in $field_array
		$sql = "SELECT distributionlist_id FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
						WHERE ";

		// Add fields to the query
		$and = '';
		foreach ($field_array as $field_name => $field_value)
		{
			$sql .= "$and$field_name = '$field_value'";
			$and = ' AND '; // Concatenate more id's
		}

		$db_result = $m2f_db->getOne($sql);;
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		// Distribution list not found, so return null
		if ($db_result == '')
	  {
  	  $m2f_error_msg = 'Distribution list id not found';
    	m2f_log(PEAR_LOG_INFO , $m2f_error_msg , __LINE__ , __FILE__ );
      return null;
	  }

		// Return it
		return $db_result;
	}

	// Returns the ids of a distribution list given a mailinglist id
	function get_distributionlist_status($id)
	{
		global $m2f_db;

		// Return a relational array
		$sql = "SELECT status FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
						WHERE distributionlist_id = '$id'";

		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		return $db_result;
	}

  // Gets information about mailing lists
  function get_mailinglists()
  {
    global $m2f_db;

    $sql = "SELECT * FROM " . _M2F_MAILINGLIST_TABLE_;
    $db_result =& $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    $mailinglists = array();
    for ($i=0;$i<count($db_result);$i++)
    {
    	$id = $db_result[$i]['mailinglist_id'];
      $mailinglists[$id]['list_email'] = $db_result[$i]['list_email'];
      $mailinglists[$id]['name'] = $db_result[$i]['name'];
      $mailinglists[$id]['status'] = $db_result[$i]['subscribe_board'];
    }

    return $mailinglists;
  }

  // Gets information about mailing lists
  function get_unlinked_mailinglists()
  {
    global $m2f_db;

    $sql = "SELECT m.mailinglist_id, m.list_email, m.name
    				FROM " . _M2F_MAILINGLIST_TABLE_ . " m
    				  LEFT JOIN " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d
    				  USING (mailinglist_id)
    				WHERE d.forum_id is NULL";
    $db_result =& $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    return $db_result;
  }

  // Gets information about distribution lists
  function get_distributionlists($distributionlist_id = null)
  {
    global $m2f_db;

    // First look for any unlinked distribution lists and delete them
    // Ideally, this would be done from phpBB when a forum is deleted. But instead
    // of making another code mod to phpBB, this should work OK.
    $sql = "SELECT d.distributionlist_id
    				FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d
    				  LEFT JOIN " . FORUMS_TABLE . " f
    				  ON d.forum_id = f.forum_id
    				WHERE f.forum_id IS NULL
    				AND d.distributionlist_id != 0"; // George added this line - this ensures the DEFAULT dist list isn't deleted
    $db_result =& $m2f_db->getCol($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    $ids_to_delete = $db_result;
    foreach ($ids_to_delete as $id)
    {
    	$this->delete_distributionlist($id);
    }

  	// Now get the real distribution lists
    $sql = "SELECT d.distributionlist_id, d.mailinglist_id, d.forum_id,
    					d.status, d.visibility, m.name, m.list_email, f.forum_name
    				FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d,
    					" . _M2F_MAILINGLIST_TABLE_ . " m, " . FORUMS_TABLE . " f
           	WHERE d.mailinglist_id = m.mailinglist_id AND d.forum_id = f.forum_id";
		if (isset($distributionlist_id))
		{
			$sql .= " AND d.distributionlist_id = " . $m2f_db->quoteSmart($distributionlist_id);
		}

    $db_result =& $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    $distributionlists = array();
    for ($i=0;$i<count($db_result);$i++)
    {
    	$id = $db_result[$i]['distributionlist_id'];
      $distributionlists[$id]['mailinglist_id'] = $db_result[$i]['mailinglist_id'];
      $distributionlists[$id]['forum_id'] = $db_result[$i]['forum_id'];
      $distributionlists[$id]['status'] = $db_result[$i]['status'];
      $distributionlists[$id]['visibility'] = $db_result[$i]['visibility'];
      $distributionlists[$id]['mailinglist_name'] = $db_result[$i]['name'];
      $distributionlists[$id]['list_email'] = $db_result[$i]['list_email'];
      $distributionlists[$id]['forum_name'] = $db_result[$i]['forum_name'];
    }

    return $distributionlists;
  }

  function get_forum_name($distributionlist_id)
  {
    global $m2f_db;

    $sql = "SELECT f.forum_name
           	FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d, " . FORUMS_TABLE . " f
           	WHERE d.forum_id = f.forum_id
           		AND d.distributionlist_id = '$distributionlist_id'";
    $db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

   	return $db_result;
  }

  // Gets subscribed mailing lists from a user id
  function get_subscribed_mailinglists($user_id)
  {
    global $m2f_db;

    $sql = "SELECT s.mailinglist_id, d.forum_id, d.distributionlist_id,
    					f.forum_name, m.name as mailinglist_name, m.list_email as mailinglist_address
    				FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d,
    					" . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . " c, 
    					" . _M2F_SUBSCRIPTION_TABLE_ . " s,
    					" . _M2F_MAILINGLIST_TABLE_ . " m,
    					" . FORUMS_TABLE . " f
    				WHERE s.mailinglist_id = d.mailinglist_id
    					AND m.mailinglist_id = d.mailinglist_id
    					AND f.forum_id = d.forum_id
    					AND s.user_id = '$user_id' 
              AND c.distributionlist_id = d.distributionlist_id 
              AND c.config_name = '_M2F_EXTERNAL_LIST_SERVER_' 
              AND c.config_value != '1' 
              AND d.visibility != 0"; 
    $db_result =& $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    return $db_result;
  }

  // Gets mailing lists which have been subscribed to because user is in a subscribed group
  function get_subscribed_mailinglists_with_group($user_id)
  {
    global $m2f_db;

    $sql = "SELECT g.group_name, f.forum_name, m.mailinglist_id, m.name as mailinglist_name, m.list_email as mailinglist_address
    				FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d,
    					" . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . " c, 
    					" . _M2F_MAILINGLIST_TABLE_ . " m,
    					" . FORUMS_TABLE . " f,
    					" . _M2F_GROUP_SUBSCRIPTION_TABLE_ . " gs,
    					" . USER_GROUP_TABLE . " ug,
    					" . GROUPS_TABLE . " g
    				WHERE ug.user_id = '$user_id'
							AND ug.group_id = gs.group_id
							AND g.group_id = ug.group_id
							AND gs.mailinglist_id = d.mailinglist_id
							AND m.mailinglist_id = d.mailinglist_id
							AND f.forum_id = d.forum_id 
							AND c.distributionlist_id = d.distributionlist_id 
							AND c.config_name = '_M2F_EXTERNAL_LIST_SERVER_' 
							AND c.config_value != '1'  
              AND d.visibility != 0"; 

    $db_result =& $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    return $db_result;
  }

	// Returns the ids of a distribution list given a mailinglist id
	function get_distributionlist_details_from_mailinglist_id($mailinglist_id)
	{
		global $m2f_db;

		// Return a relational array
		$sql = "SELECT * FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
						WHERE mailinglist_id = '$mailinglist_id'";

		$db_result =& $m2f_db->getRow($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		return $db_result;
	}

  // Gets unsubscribed mailing lists from a user id
  function get_unsubscribed_mailinglists($user_id)
  {
    global $m2f_db;

    $distributionlists = $this->get_distributionlists();
    $i = 0;
    foreach ($distributionlists as $id => $distributionlist)
    {
      if ($distributionlist['visibility'] == '0') continue;

      $sql = "SELECT d.mailinglist_id
      				FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d,
      					" . _M2F_SUBSCRIPTION_TABLE_ . " s
      				WHERE d.distributionlist_id = '$id'
      					AND s.mailinglist_id = d.mailinglist_id
      					AND s.user_id = '$user_id'";
      $db_result = $m2f_db->getOne($sql);
	    check_db_result($db_result, $sql, __LINE__, __FILE__);

     	if ($db_result != '')
     	{
     		$matched_mailinglist_ids[] = $db_result;
     	}

			$sql = "SELECT d.mailinglist_id 
							FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d, 
								" . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . " c 
							WHERE d.distributionlist_id = '$id' 
								AND c.distributionlist_id = d.distributionlist_id 
								AND c.config_name = '_M2F_EXTERNAL_LIST_SERVER_' 
								AND c.config_value <> '1' ";       
			$db_result = $m2f_db->getOne($sql);
	    check_db_result($db_result, $sql, __LINE__, __FILE__);

     	if ($db_result != '')
     	{
     		$possible_mailinglist_ids[] = $db_result;
     	}
    }

		if (is_array($possible_mailinglist_ids))
		{
			$mailinglist_ids = is_array($matched_mailinglist_ids) ? array_diff($possible_mailinglist_ids, $matched_mailinglist_ids) : $possible_mailinglist_ids;
		}

		if (!is_array($mailinglist_ids))
		{
			return;
		}

		$i = 0;
		foreach($mailinglist_ids as $mailinglist_id)
		{
      $sql = "SELECT m.mailinglist_id, f.forum_name, f.forum_id,
      					m.name as mailinglist_name, m.list_email as mailinglist_address
      				FROM " . _M2F_MAILINGLIST_TABLE_ . " m,
      					" . FORUMS_TABLE . " f,
      					" . _M2F_DISTRIBUTIONLIST_TABLE_ . " d
							WHERE d.mailinglist_id = '$mailinglist_id'
								AND m.mailinglist_id = '$mailinglist_id'
								AND f.forum_id = d.forum_id";

      $db_result = $m2f_db->getRow($sql);
	    check_db_result($db_result, $sql, __LINE__, __FILE__);
    	$mailinglists[$i] = $db_result;
    	$i++;
		}
    return $mailinglists;
  }

  function get_mailinglist_address($distributionlist_id)
  {
    global $m2f_db;
  	$sql = "SELECT m.list_email
  					FROM " . _M2F_MAILINGLIST_TABLE_ . " m,
  						" . _M2F_DISTRIBUTIONLIST_TABLE_ . " d
  					WHERE d.distributionlist_id = '$distributionlist_id'
  						AND m.mailinglist_id = d.mailinglist_id";
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

  	return $db_result;
  }

  function get_mailinglist_name($distributionlist_id)
  {
    global $m2f_db;
  	$sql = "SELECT m.name
  					FROM " . _M2F_MAILINGLIST_TABLE_ . " m,
  						" . _M2F_DISTRIBUTIONLIST_TABLE_ . " d
  					WHERE d.distributionlist_id = '$distributionlist_id'
  						AND m.mailinglist_id = d.mailinglist_id";
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

  	return $db_result;
  }

  // Gets information about forums
	function get_forums()
  {
    return $this->m2f_forum->get_forums();
  }

	// Pulls mails from the mailinglist, and inserts them into the forum
	// This should be called from a crontab script
	function pull_mailinglist_msgs($distributionlist_id)
	{
		$import_results = array($distributionlist_id => array('IMPORTED' => 0, 'IGNORED' => 0, 'POSTED' => 0));

		// Get the info of the distribution list
		$distributionlist_ids = $this->distributionlist_ids($distributionlist_id);

		// Get the distributionlist config
		$params = $this->get_distributionlist_configuration($distributionlist_id);

		// Get the source mailinglist_id from the distribution list table
		$forum_id = $distributionlist_ids['forum_id'];

		// Get the target forum_id from the distribution list table
		$mailinglist_id = $distributionlist_ids['mailinglist_id'];

		// Retrieve the mails!
		$raw_mail_msgs = $this->m2f_mailinglist->retrieve_msgs($params , $mailinglist_id);

		$processed_msgs = array();

		// Insert them into the forum
		foreach ($raw_mail_msgs as $message_key => $raw_mail_msg)
		{
			$raw_mail_msg = trim($raw_mail_msg);
			if (empty($raw_mail_msg)) continue;
			
			$processed_msgs[] = $message_key;

			// Make mail msg
			$mail_msg = new mail_msg();

			// decode the MIME of the raw message, adding the parts to the mail_msg object
			$this->m2f_mailinglist->decode_raw_message($raw_mail_msg, $mail_msg);

			// Get the message ID and the md5 hash of the email. Useful for preventing looping messages
			$this->md5_hash = md5($raw_mail_msg); // this is a quick and dirty way to make the hash avaiable to other functions. Sorry!   George
			$message_id = array_key_exists('message-id', $mail_msg->headers) ? $mail_msg->headers['message-id'] : '';

			list($uniq_id, $tag, $is_new) = $this->check_message_id($this->md5_hash, $message_id);

			if ($tag == 'CUR' && !$is_new) // a message which already had a hash entry, but whose tag is 'CUR' must either be currently importing, or the result of a m2f crash
			{
				global $m2f_db;
				$sql = "SELECT logtime FROM " . _M2F_MESSAGE_HASH_TABLE_ . 
								" WHERE id = '$uniq_id'";
				$old_logtime = $m2f_db->getOne($sql);
				check_db_result($old_logtime, $sql, __LINE__, __FILE__);

				if ((time() - $old_logtime) < 600) // within 10 minutes - assume it's still importing
				{
					m2f_log(PEAR_LOG_INFO , 'Incoming mail is being imported by another script, ignoring it' , __LINE__ , __FILE__ );
					continue;
				}
			}

			switch ($tag)
			{
				case 'IMP': // message has been imported already
					m2f_log(PEAR_LOG_INFO , 'Incoming mail "' . $mail_msg->headers['subject'] . '"  from "' . $mail_msg->headers['from'] . '" failed md5 hash or message-id test (i.e. it has already been imported), skipping it' , __LINE__ , __FILE__ );
					break;

				case 'REJ': // message has been rejected before
					m2f_log(PEAR_LOG_INFO , 'Incoming mail "' . $mail_msg->headers['subject'] . '"  from "' . $mail_msg->headers['from'] . '" was previously rejected, skipping it' , __LINE__ , __FILE__ );
					break;

				case 'SNT': // message was sent by m2f
					m2f_log(PEAR_LOG_INFO , 'Incoming mail was sent by M2F, skipping it' , __LINE__ , __FILE__ );
					$this->set_tag($uniq_id, 'REJ');
					$import_results[$distributionlist_id]['IGNORED']++;
					break;
			}
			
			if ($tag == 'IMP' || $tag == 'REJ' || $tag == 'SNT') continue;

			
			// check the message isn't larger than the maximum for this dist list
			// this check done early to avoid wasting resources on spam
			$max_size = $params['_M2F_MAX_MESSAGE_SIZE_'] * 1024;
			$msg_size = strlen($raw_mail_msg);

			if ($max_size && (strlen($raw_mail_msg) > $max_size))
			{
				m2f_log(PEAR_LOG_INFO , 'Incoming mail was too large. Mail size = ' . $msg_size . ' bytes, maximum for this dist list is ' . $max_size . ' bytes. (1 kB = 1024 bytes)', __LINE__ , __FILE__ );
				$this->refuse_post($params, $mail_msg, 'size', $distributionlist_id, $mailinglist_id);
				$this->set_tag($uniq_id, 'REJ');
				$import_results[$distributionlist_id]['IGNORED']++;
				continue;
			}


			// Run spam email patterns
			if (($blocked = $this->run_block_patterns($mail_msg, $distributionlist_id)) !== FALSE)
			{
				m2f_log(PEAR_LOG_INFO , 'Incoming mail matched a spam-block pattern: "' . $blocked[1] . '" in the ' . $blocked[0], __LINE__ , __FILE__ );
				if ($params['_M2F_DISALLOW_GUEST_ACTION_'] == 'moderate')
				{
					$this->new_mail_event($mail_msg, $distributionlist_id, $params, TRUE);
					$this->set_tag($uniq_id, 'IMP');
					$import_results[$distributionlist_id]['MODERATED']++;
					$import_results[$distributionlist_id]['IMPORTED']++;
				}
				else
				{
					$this->refuse_post($params, $mail_msg, 'spam', $distributionlist_id, $mailinglist_id, 'Pattern "' . $blocked[1] . '" was matched in the message\'s ' . $blocked[0]);
					$this->set_tag($uniq_id, 'REJ');
					$import_results[$distributionlist_id]['IGNORED']++;
				}
				
				continue;
			}

			if ($params['_M2F_IGNORE_TO_HEADER_'] == 1)
			{
				$result = $this->new_mail_event($mail_msg, $distributionlist_id, $params);
				if ($result === TRUE) $import_results[$distributionlist_id]['POSTED']++;
				else if ($result === 'mod') $import_results[$distributionlist_id]['MODERATED']++;
				else if ($result === FALSE) $import_results[$distributionlist_id]['IGNORED']++;
			}
			else
			{
				m2f_log(PEAR_LOG_DEBUG , "To line, before: " . $mail_msg->headers['to'] , __LINE__ , __FILE__ );
				m2f_log(PEAR_LOG_DEBUG , "Cc line, before: " . $mail_msg->headers['cc'] , __LINE__ , __FILE__ );
				$to_array = array();
				$cc_array = array();

				// Finds all email addresses in a string
				$match_pattern = '/[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)*\@[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+(?:\.[^\x00-\x20()<>@,;:\\".[\]\x7f-\xff]+)+/i';

				if (preg_match_all($match_pattern, $mail_msg->headers['to'], $matches))
				{
					$to_array = $matches[0];
					m2f_log(PEAR_LOG_DEBUG , "To line: ", __LINE__ , __FILE__ , $to_array);
				}
				if (preg_match_all($match_pattern, $mail_msg->headers['cc'], $matches))
				{
					$cc_array = $matches[0];
					m2f_log(PEAR_LOG_DEBUG , "Cc line: ", __LINE__ , __FILE__ , $cc_array);
				}

				// Clean double addresses
				$address_array = array_unique(array_merge($to_array, $cc_array));

				m2f_log(PEAR_LOG_DEBUG , "Discovered these addresses: " , __LINE__ , __FILE__ , $address_array);

				foreach ($address_array as $address)
				{
					m2f_log(PEAR_LOG_DEBUG , "Using: " . $address , __LINE__ , __FILE__ );
					$mail_msg->headers['to'] = $address;
					$mailinglist_id = $this->get_mailinglist_id($mail_msg);
					
					if ($mailinglist_id == 0) // If the email address wasn't found...
					{
						$m2f_error_msg = 'The \'To:\' email address - \'' . $mail_msg->headers['to'] . '\' - was not found in the mailinglist table. Message ignored.';
						m2f_log(PEAR_LOG_INFO , $m2f_error_msg , __LINE__ , __FILE__ );
						$import_results[$distributionlist_id]['IGNORED']++;
						continue;
					}
					
					$temp_distributionlist_id = $this->get_distributionlist_id(array('mailinglist_id' => $mailinglist_id));
					$status = $this->get_distributionlist_status($temp_distributionlist_id);
					if ($status)
					{
						$result = $this->new_mail_event($mail_msg, $temp_distributionlist_id, $params);
						// @ to supress errors, in case the results array for this dist_list hasn't been created yet
						if ($result === TRUE) @$import_results[$temp_distributionlist_id]['POSTED']++; 
						else if ($result === 'mod') @$import_results[$temp_distributionlist_id]['MODERATED']++;
						else if ($result === FALSE) @$import_results[$temp_distributionlist_id]['IGNORED']++;
					}
				}
			}
			
			$import_results[$distributionlist_id]['IMPORTED']++;
			$this->set_tag($uniq_id, 'IMP');
		}

		$this->clean_up_messsages($processed_msgs, $params);
		return array('results' => $import_results, 'max_import' => $params['_M2F_MAX_MSG_RETRIEVE_']);
	}

	function clean_up_messsages($processed_msgs, $params)
	{
		if (!empty($processed_msgs) && $params['_M2F_DELETE_IMPORTED_MESSAGES_'] == 1)
		{
			switch ($params['_M2F_INPUT_BACKEND_'])
			{
				case 'mbox':

					// test the defaut system tmp directory for suitability
					$umaskOld = umask(077);
					$ftempname = tempnam ('', rand(0, 9));

					$ftemp = fopen($ftempname, "w");

					if ($ftemp == false) // not suitable
					{
						// test the M2F_ROOT_PATH directory for suitability as a tmp directory
						$ftempname = tempnam (M2F_ROOT_PATH, rand(0, 9));
						$ftemp2 = fopen($ftempname, "w");

						if ($ftemp2 == false)  // not suitable
						{
							m2f_log(PEAR_LOG_CRIT , "Cannot create a temporary file at " . realpath(M2F_ROOT_PATH) . ". Please make this directory writeable by the webserver.", __LINE__ , __FILE__ );
						}
						else
						{
							fclose($ftemp2);
							unlink($ftempname);
							define ('_MBOX_TMP_DIR_', M2F_ROOT_PATH);
						}
					}
					else
					{
						fclose($ftemp);
						unlink($ftempname);
						define ('_MBOX_TMP_DIR_', '');
					}

					$result = $this->m2f_mailinglist->mbox->removeProcessedmessages($this->m2f_mailinglist->mbox_id);
					if (PEAR::isError($result))
					{
						m2f_log(PEAR_LOG_CRIT , "Failed deleting messages from mbox. " . $result->getMessage(), __LINE__ , __FILE__ );
					}
					umask($umaskOld);
					break;

				case 'pop3':
					foreach ($processed_msgs as $msg_number)
					{
						$result = $this->m2f_mailinglist->pop3->deleteMsg($msg_number);
						if (PEAR::isError($result))
						{
							m2f_log(PEAR_LOG_CRIT , "Failed to delete message $msg_number from POP box. " . $result->getMessage(), __LINE__ , __FILE__ );
						}
					}
					break;
					
				case 'gmail':
					foreach ($processed_msgs as $msg_number)
					{
						$this->m2f_mailinglist->gmail->performAction(15, $msg_number);
					}
					break;
			}
		}

		switch ($params['_M2F_INPUT_BACKEND_'])
		{
			case 'mbox':
				$result = @unlink($params['_M2F_MBOX_FILE_'] . '.lock');
				if (!$result)
				{
					m2f_log(PEAR_LOG_NOTICE , "Failed deleting mbox lock file.", __LINE__ , __FILE__ );
					return;
				}
				break;

			case 'pop3':
				$this->m2f_mailinglist->pop3->disconnect();
				break;

			case 'gmail':
				$this->m2f_mailinglist->gmail->disconnect();
				break;
		}
	}

	function check_message_id($md5, $id)
	{
	  global $m2f_db;
	  $is_new = TRUE;
		$tag = 'CUR';
	  
		$hash = empty($id) ? $md5 : $id;
		$id = $m2f_db->nextId(_M2F_MESSAGE_HASH_TABLE_, TRUE);
		
		$sql = "INSERT INTO " . _M2F_MESSAGE_HASH_TABLE_ .
						" (hash, logtime, tag, id)".
						" VALUES ('" . addcslashes($hash,"'") . "', '" . time() . "', 'CUR', '$id')";
    $db_result = $m2f_db->query($sql); // don't check for error here - there may well be one as 'hash' is a unique key
    
    if (DB::isError($db_result)) // this message has already been processed, so get the id and tag
    {	
			$sql = "SELECT id, tag FROM " . _M2F_MESSAGE_HASH_TABLE_ . 
							" WHERE hash = '" . addcslashes($hash,"'") . "'";
			$db_result = $m2f_db->getRow($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			$tag = $db_result['tag'];
			$id = $db_result['id'];
			$is_new = FALSE;
    }
		
		return array($id, $tag, $is_new); // return unique message id
 	}

	function set_tag($id, $tag)
	{
	  global $m2f_db;
	  
		$sql = "UPDATE " . _M2F_MESSAGE_HASH_TABLE_ .
						" SET tag = '$tag' WHERE id = '$id'";
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
		return $db_result;
	}
	
/*
	function get_tag($id)
	{
	  global $m2f_db;
	  
		$sql = "SELECT tag FROM " . _M2F_MESSAGE_HASH_TABLE_ . 
						" WHERE id = '$id'";
		$db_result = $m2f_db->getOne($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
		return $db_result;
	}
*/

	// Deletes a distributionlist
	function delete_distributionlist ($distributionlist_id)
	{
	  global $m2f_db;
		if ($distributionlist_id == 0) return; // don't delete default list!

		// Delete the distributionlist
		$sql = "DELETE FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
						WHERE distributionlist_id = '$distributionlist_id'";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    // Delete the distributionlist config
		$sql = "DELETE FROM " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
						WHERE distributionlist_id = '$distributionlist_id'";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    // Delete the distributionlist config
		$sql = "DELETE FROM " . _M2F_DIST_LIST_PATTERNS_TABLE_ . "
						WHERE distributionlist_id = '$distributionlist_id'";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	}

	// Makes a new distributionlist, and returns a proper distributionlist_id
	// Params are equivalent to columns: status, name, etc
	function new_distributionlist ($params)
	{
	  global $m2f_db;
    include_once(M2F_ROOT_PATH."db/m2f_schema.php");
    $distributionlist_id = m2f_create_distributionlist();

    // Set new distributionlist according to parameters
		$this->set_distributionlist($distributionlist_id, $params);

		return $distributionlist_id;
	}

	// Change distributionlist setting
	function set_distributionlist($distributionlist_id, $params)
	{
	  global $m2f_db;

		// Foreach column in m2f_distributionlist table, update with $params
		// todo: Put default values if !isset($params['column_name'])

		// Update the distribution list information
		$sql = "UPDATE " . _M2F_DISTRIBUTIONLIST_TABLE_ . " SET ";
		$comma = '';
		if (isset($params['mailinglist_id']))
		{
		  $sql .= "mailinglist_id = '{$params['mailinglist_id']}' ";
		  $comma = ',';
		}
		if (isset($params['forum_id']))
		{
		  $sql .= $comma . "forum_id = '{$params['forum_id']}' ";
		  $comma = ',';
		}
		if (isset($params['status']))
		{
			$sql .= $comma . "status='{$params['status']}' ";
		}
		if (isset($params['visibility']))
		{
			$sql .= $comma . "visibility='{$params['visibility']}' ";
		}
		$sql .= " WHERE distributionlist_id = '$distributionlist_id'";

		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	}

	//Configures a distributionlist
	function configure_distributionlist($distributionlist_id, $params)
	{
	  global $m2f_db;

		// Foreach column in m2f_distributionlist_config table, update with $params
		// todo: Put default values if !isset($params['column_name'])

		// Clean up current config
		$sql = "DELETE FROM " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
							WHERE distributionlist_id = '$distributionlist_id'";
    $db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		// Update the distribution list information
		foreach ($params as $key => $value)
		{
		  if (!is_array($value))
		  {
		    $value = array($value);
		  }

      foreach ($value as $univalue)
      {
        $sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
        					(distributionlist_id , config_name, config_value)
        				VALUES ('$distributionlist_id', '$key', '" . $m2f_db->escapeSimple($univalue) . "')";
        $db_result = $m2f_db->query($sql);
		    check_db_result($db_result, $sql, __LINE__, __FILE__);
      }
		}
	}

	// Returns distributionlist configuration
	function get_distributionlist_configuration($distributionlist_id)
	{
	  global $m2f_db;
	  $sql = "SELECT config_name, config_value
	  				FROM " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
	  				WHERE distributionlist_id = '$distributionlist_id'";
	  $db_result =& $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

	  // Populate $params[] array
    $params = array();

    // Fix config directives that should always be array
    $params['_M2F_REPLY_PREFIXES_'] = array();

    // Import config parameters...
    foreach ($db_result as $row)
    {
    	$config_name = $row['config_name'];
    	$config_value = $row['config_value'];

      // Detect if its a multi-valued config variable
      if (isset($params[$config_name]))
      {
        // If it isnt already treated as an array..
        if (!is_array($params[$config_name]))
        {
          // Not an array yet, build one
          $old_value = $params[$config_name];
          $params[$config_name] = array();
          $params[$config_name][] = $old_value;
        }

        // Array set up, add new config value
        $params[$config_name][] = $config_value;
      }
      else
      {
        // Uni-valued config variable
        $params[$config_name] = $config_value;
      }
    }

	  return $params;
	}

	// Changes a config directive

	function run_patterns(&$forum_msg, $distributionlist_id)
	{
		// First, remove any of the m2f signatures we can find
		if (_M2F_SIGNATURE_SEPARATOR_ != '')
		{
			$sig_div = preg_quote(_M2F_SIGNATURE_SEPARATOR_, '/');
			
			$search = '/((^[ >]*>$\n)*((^[ >]*>$\n){2}))?^[ >]*?' . $sig_div . '.*?$(.|\n)*?^[ >]*?' . $sig_div . '.*?$\n((^[ >]*>$\n)*(?=(^[ >]*>$\n){2}))?/ms';
									// This matches:
									// An optional ( any number of cells followed by exactly two cells (which become subpattern $3 and are replaced) )
									// a start of line, any number of spaces or '>', the sig_div, non-greedy anything, a start of line, any number of spaces or '>', the sig_div, anything till end of line
									// an optional ( any number of cells followed by exactly two cells (which are not matched) )
									// ... when 1 'cell' is (a start of line, any number of spaces or '>', at least one '>', an dend of line and a newline character)
									
			$replace = "$3";
			$forum_msg->body = preg_replace($search , $replace, $forum_msg->body);
		}

		if (_M2F_FORUM_POST_SIGNATURE_ != '')
		{
			$sig = preg_quote(_M2F_FORUM_POST_SIGNATURE_, '/');
			$search = '/^([\s>]*)' . $sig . '/m';
			$forum_msg->body = preg_replace($search , '', $forum_msg->body);
		}

		$patterns = $this->get_linked_patterns($distributionlist_id, 'search');

		foreach ($patterns as $pattern)
		{
			if ($pattern['use_regexp'])
			{
				if ($pattern['target'] & _M2F_TARGET_BODY_)
				{
					$forum_msg->body = preg_replace('/' . $pattern['pattern_match'] . '/' . $pattern['modifiers'], $pattern['pattern_replace'], $forum_msg->body);
				}
				if ($pattern['target'] & _M2F_TARGET_SUBJECT_)
				{
					$forum_msg->title = preg_replace('/' . $pattern['pattern_match'] . '/' . $pattern['modifiers'], $pattern['pattern_replace'], $forum_msg->title);
				}
			}
			else
			{
				if ($pattern['target'] & _M2F_TARGET_BODY_)
				{
					$forum_msg->body = str_replace($pattern['pattern_match'], $pattern['pattern_replace'], $forum_msg->body);
				}
				if ($pattern['target'] & _M2F_TARGET_SUBJECT_)
				{
					$forum_msg->title = str_replace($pattern['pattern_match'], $pattern['pattern_replace'], $forum_msg->title);
				}
			}
		}
		$forum_msg->body = trim($forum_msg->body);
		$forum_msg->title = trim($forum_msg->title);
		return TRUE;
	}

	
	// $mode is either 'search' or 'block'
	function get_linked_patterns($dist_id, $mode)
	{
		global $m2f_db;

		$sql = 'SELECT * FROM  ' . _M2F_PATTERN_MATCH_TABLE_ . ' p, ' . _M2F_DIST_LIST_PATTERNS_TABLE_ . ' dp
			 WHERE dp.distributionlist_id = ' . $dist_id . '
			 AND p.pattern_id = dp.pattern_id
			 AND p.purpose = \'' . $mode . '\'
			 ORDER BY dp.pattern_order';

		$db_result = $m2f_db->getAll($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);

		return $db_result;
	}
	
	
	function run_header_block_patterns($headers, $pattern, $use_regexp, &$matches, $key_override = NULL)
	{
		foreach ($headers as $key => $value)
		{
			if ($key_override) $key = $key_override.'['.$key.']';
			if (is_array($value)) 
			{
				$this->run_header_block_patterns($value, $pattern, $use_regexp, $matches, $key);
				continue;
			}
			if ($use_regexp && preg_match($pattern, $value)) $matches[] = $key;
			else if (!$use_regexp && (strpos($value, $pattern) !== FALSE)) $matches[] = $key;
		}
	}

	function run_block_patterns(&$mail_msg, $distributionlist_id)
	{
		$patterns = $this->get_linked_patterns($distributionlist_id, 'block');
		$match = FALSE;
		$match_loc = '';

		foreach ($patterns as $pattern)
		{
			if ($pattern['use_regexp'])
			{
				if ($pattern['target'] & _M2F_TARGET_BODY_)
				{
					if (preg_match('/' . $pattern['pattern_match'] . '/' . $pattern['modifiers'], $mail_msg->html_body)) 
					{
						$match = TRUE;
						$match_loc = 'HTML Body';
						break;
					}
					if (preg_match('/' . $pattern['pattern_match'] . '/' . $pattern['modifiers'], $mail_msg->body)) 
					{
						$match = TRUE;
						$match_loc = 'Body';
						break;
					}
				}
				if ($pattern['target'] & _M2F_TARGET_SUBJECT_)
				{
					if (preg_match('/' . $pattern['pattern_match'] . '/' . $pattern['modifiers'], $mail_msg->headers['subject'])) 
					{
						$match = TRUE;
						$match_loc = 'Subject';
						break;
					}
				}
				if ($pattern['target'] & _M2F_TARGET_HEADERS_)
				{
					$matches = array();
					$this->run_header_block_patterns($mail_msg->headers, '/' . $pattern['pattern_match'] . '/' . $pattern['modifiers'], TRUE, $matches);
					if (!empty($matches)) 
					{
						$match = TRUE;
						$match_loc = 'Header ' . $matches[0];
						break;
					}
				}
			}
			else
			{
				if ($pattern['target'] & _M2F_TARGET_BODY_)
				{
					if (strpos($mail_msg->html_body, $pattern['pattern_match']))
					{
						$match = TRUE;
						$match_loc = 'HTML Body';
						break;
					}
					if (strpos($mail_msg->body, $pattern['pattern_match']))
					{
						$match = TRUE;
						$match_loc = 'Body';
						break;
					}
				}
				if ($pattern['target'] & _M2F_TARGET_SUBJECT_)
				{
					if (strpos($mail_msg->headers['subject'], $pattern['pattern_match']) !== FALSE)
					{
						$match = TRUE;
						$match_loc = 'Subject';
						break;
					}
				}
				if ($pattern['target'] & _M2F_TARGET_HEADERS_)
				{
					$matches = array();
					$this->run_header_block_patterns($mail_msg->headers, $pattern['pattern_match'], FALSE, $matches);
					if (!empty($matches)) 
					{
						$match = TRUE;
						$match_loc = 'Header ' . $matches[0];
						break;
					}
				}
			}
		}

		if ($match) $match_pattern = $pattern['description'];
		return $match ? array($match_loc, $match_pattern) : FALSE;
	}


	// Import msgs of active distribution lists
	function import_msgs()
	{
		global $m2f_db;

		m2f_log(PEAR_LOG_NOTICE , "*********************************\n** Starting m2f message import **\n*********************************", __LINE__ , __FILE__ );

		// Check to make sure board is enabled
		$sql = "SELECT config_value FROM " . CONFIG_TABLE . "
						WHERE config_name = 'board_disable'";
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		if ($db_result && !defined("IN_ADMIN"))
		{
      m2f_log(PEAR_LOG_ERROR, 'Cannot import. Board is disabled.', __LINE__ , __FILE__ );
      return;
		}

		// check if distributionlist is specified as a command line argument
		$parameter = isset($_SERVER['argv'][1]) ? $_SERVER['argv'][1] : null;
		if (! isset($parameter))
		{
			// nothing specified on command line
			$distributionlist_id = null;
		}
		else
		{
			m2f_log(PEAR_LOG_DEBUG, "Looking for distributionlist specified by parameter \"$parameter\"", __LINE__, __FILE__);
			if (preg_match('#^[0-9]+$#', $parameter))
			{
				// an integer value was specified - check if it is a valid distributionlist id
				$sql = "SELECT COUNT(*) FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
								WHERE distributionlist_id = " . $parameter; // no need to quote because of regex match above
				$db_result = $m2f_db->getOne($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
				if ($db_result == 0)
				{
					m2f_log(PEAR_LOG_WARNING, 'Specified distributionlist id "' . $parameter . '" doesn\'t exist!', __LINE__, __FILE__);
					$distributionlist_id = null;
				}
				else
				{
					// valid distributionlist id specified
					m2f_log(PEAR_LOG_DEBUG, "Distributionlist id \"$parameter\" found to be valid", __LINE__, __FILE__);
					$distributionlist_id = (int) $parameter;
				}
			}
			else
			{
				// parameter specifies not the id from the db but the mailinglist address
				$sql = "SELECT d.distributionlist_id
								FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . " d,
							 		" . _M2F_MAILINGLIST_TABLE_ . " m
							 	WHERE d.mailinglist_id = m.mailinglist_id
							 	  AND m.list_email = " . $m2f_db->quoteSmart($parameter);
				$db_result = $m2f_db->getOne($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
				if (! $db_result)
				{
					m2f_log(PEAR_LOG_WARNING, 'Specified mailinglist address "' . $parameter . '" not found!', __LINE__ , __FILE__ );
					$distributionlist_id = null;
				}
				else
				{
					// found distributionlist id connected to specified mailinglist address
					m2f_log(PEAR_LOG_DEBUG, "Found distributionlist id \"$db_result\" via specified list address \"$parameter\"", __LINE__, __FILE__);
					$distributionlist_id = $db_result;
				}
			}
		}

		$lists = $this->get_distributionlists($distributionlist_id);
		$import_results = array();
		$grand_total = array();
		$imported = 0;
		$ignored = 0;
		$posted = 0;
		$moderated = 0;

    foreach ($lists as $list_id => $list_details)
    {
    	$this->list_id = $list_id; // set this so the log can access it
      if ($list_details['status'] == 1)
      {
        m2f_log(PEAR_LOG_INFO, 'Importing messages from distribution list:' . $list_id, __LINE__ , __FILE__ );
        $import_results[$list_id] = $this->pull_mailinglist_msgs($list_id);
      }
      else
      {
        m2f_log(PEAR_LOG_INFO, 'Inactive distribution list: ' . $list_id, __LINE__ , __FILE__ );
      }
    }
 		
 		unset($this->list_id);
 		
 		foreach ($import_results as $list_id => $results)
 		{
 			$grand_total[$list_id]['MAX_IMPORT'] = $results['max_import'];
 			foreach ($results['results'] as $sub_list_id => $sub_results)
 			{	
 				@$grand_total[$sub_list_id]['IMPORTED'] += $sub_results['IMPORTED'];
 				$imported += $sub_results['IMPORTED'];
 				
 				@$grand_total[$sub_list_id]['IGNORED'] += $sub_results['IGNORED'];
 				$ignored += $sub_results['IGNORED'];

 				@$grand_total[$sub_list_id]['MODERATED'] += $sub_results['MODERATED'];
 				$moderated += $sub_results['MODERATED'];
 				
 				@$grand_total[$sub_list_id]['POSTED'] += $sub_results['POSTED'];
 				if ($results['moderated'] == '1')
 				{
 					$moderated += $sub_results['POSTED'];
 				}
 				else
 				{
 					$posted += $sub_results['POSTED'];
 				}
 			}
 		}
 		
 		if (!empty($grand_total)) $human_results = "\tlist\t\timported\tignored\t\tposted\t\tmoderated\n";
 		foreach ($grand_total as $list => $result) 
 		{
 			$human_results .= "\t $list\t\t {$result['IMPORTED']}\t\t {$result['IGNORED']}\t\t {$result['POSTED']}\t\t {$result['MODERATED']}\n";
 		}
 		m2f_log(PEAR_LOG_NOTICE , "Imported $imported good message(s). Ignored $ignored message(s). Posted $posted message(s). Moderated $moderated message(s):\n\n" . $human_results, __LINE__ , __FILE__ );

		$grand_total['time'] = round((array_sum(explode(" ",microtime())) - $this->start_time),2);

 		m2f_log(PEAR_LOG_NOTICE , 'Finished. Import took ' . $grand_total['time'] . ' seconds.' , __LINE__ , __FILE__ );
 		return $grand_total; // the return value is only user on the manual force import from Admin CP
  }

}
?>